/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/**
 * 	ctmpl/hashtable.h
 *		written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 * 		$Id: hashtable.h,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *	<Hashtable Template>
 */

typedef struct PKG_NAME_(ht) *PKG_NAME_(ht);
typedef struct PKG_NAME_(ht_ctl) PKG_NAME_(ht_ctl);
typedef struct PKG_NAME_(ht_iterator) PKG_NAME_(ht_iterator);
typedef struct PKG_NAME_(ht_iterator_data) 
			*PKG_NAME_(ht_iterator_data); 
typedef struct PKG_NAME_(ht_iterator_data) *PKG_NAME_(htnode);

/* slist.h */
#define PKG_SETVAL_T	 PKG_NAME_(htnode)
#include "ctmpl/slist.h"
#undef  PKG_SETVAL_T

#if !defined(PKG_MAPKEY_T) || !defined(PKG_MAPVAL_T)
#error "PKG_MAPKEY_T & PKG_MAPVAL_T must be defined."
#else
typedef PKG_MAPKEY_T PKG_NAME_(htkey_t);
typedef PKG_MAPVAL_T PKG_NAME_(htval_t);
#endif /* !defined(PKG_MAPKEY_T) || !defined(PKG_MAPVAL_T) */

/*	
 *  PKG_NAME_(ht_hash_func_t):
 *	this function generates integer key from value.
 *	returned integer can be greater than size of hashtable.
 */
typedef unsigned int (*PKG_NAME_(ht_hash_func_t))(PKG_NAME_(htkey_t));

/*	
 *  PKG_NAME_(ht_cmpkey_func_t):
 *	this function compares two keys.
 */
typedef int (*PKG_NAME_(ht_cmpkey_func_t))
	(PKG_NAME_(htkey_t), PKG_NAME_(htkey_t));

/*
 *  htnodevalue_access_func_t
 *	this function deletes value of hashtable node. 
 */
typedef void  (*PKG_NAME_(htnode_access_func_t))(PKG_NAME_(htkey_t), 	
					         PKG_NAME_(htval_t));

/*
 *  htnodevalue_foreach_func_t
 */
typedef void  (*PKG_NAME_(htnode_foreach_func_t))(PKG_NAME_(htkey_t), 	
					          PKG_NAME_(htval_t),
						 void *);


/*
 *  htnode_duplicate_func_t
 *	this function duplicates hashtable node. 
 */
typedef struct PKG_NAME_(ht_iterator_data)  
	(*PKG_NAME_(htnode_duplicate_func_t))(PKG_NAME_(htkey_t), 	
					      PKG_NAME_(htval_t));

/* iterator for hashtable */
struct PKG_NAME_(ht_iterator){
	short			iter_type;  /* iterator type */
	short			iter_subtype;  /* iterator subtype */
	PKG_NAME_(ht) 		iter_aggr;  /* iterator aggregate */
	PKG_NAME_(slistnode) 	iter_item;  /* iterator item */
};

/* iterator controller for hashtet */
struct PKG_NAME_(ht_ctl){
	PKG_NAME_(ht_iterator) enditr;
	void		       (*delete_by)(PKG_NAME_(ht), 
				 PKG_NAME_(htnode_access_func_t));
	void		       (*clear_by)(PKG_NAME_(ht), 
				 PKG_NAME_(htnode_access_func_t));
	PKG_NAME_(ht) 	       (*dup_by)(PKG_NAME_(ht), 
				 PKG_NAME_(htnode_duplicate_func_t));
	void		       (*foreach)(PKG_NAME_(ht), 
				 PKG_NAME_(htnode_foreach_func_t),
				 void *);
	iterator_cnt_t	       (*count)(PKG_NAME_(ht));
	PKG_NAME_(ht_iterator) (*first)(PKG_NAME_(ht));
	PKG_NAME_(ht_iterator) (*last)(PKG_NAME_(ht));
	PKG_NAME_(ht_iterator) (*at)(PKG_NAME_(ht), iterator_pos_t);
	PKG_NAME_(ht_iterator) (*next)(PKG_NAME_(ht_iterator));
	PKG_NAME_(ht_iterator) (*prev)(PKG_NAME_(ht_iterator));
	PKG_NAME_(ht_iterator) (*erase_gonext)(PKG_NAME_(ht_iterator));
	PKG_NAME_(ht_iterator) (*erase_goprev)(PKG_NAME_(ht_iterator));
	int			(*is_equal)(PKG_NAME_(ht_iterator), 
					    PKG_NAME_(ht_iterator));
	PKG_NAME_(ht_iterator_data) 
				(*retrieve)(PKG_NAME_(ht_iterator));
	PKG_NAME_(ht_iterator) 
				(*find)(PKG_NAME_(ht), PKG_NAME_(htkey_t));
	void 			(*insert_pair)(PKG_NAME_(ht), 
					       PKG_NAME_(htkey_t),
					       PKG_NAME_(htval_t));
};

struct PKG_NAME_(ht_iterator_data){
	PKG_NAME_(htkey_t) first;	/* key, will not be free */
	PKG_NAME_(htval_t) second;	/* value */
};

struct PKG_NAME_(ht) {
	PKG_NAME_(slist) *table;	/* table */
	int size;	/* table size */
	int count;	/* number of contained value */
	PKG_NAME_(ht_hash_func_t) hashkey;  /* hash key function */
	PKG_NAME_(ht_cmpkey_func_t)  cmpkey;  /* compare key function */
	m_alloc_func_t 	alloc;	    /* malloc function */
	m_free_func_t 	free;	    /* free function */

	/* iterator controller */
	PKG_NAME_(ht_ctl) *ctl;
};

/* prototypes */
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(ht) 
PKG_NAME_(ht_new)(int, 	/* table size */
		  PKG_NAME_(ht_hash_func_t), 
		  PKG_NAME_(ht_cmpkey_func_t));

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(ht) 
PKG_NAME_(ht_new_by)(int, 
		     PKG_NAME_(ht_hash_func_t), 
		     PKG_NAME_(ht_cmpkey_func_t),
	 	     m_alloc_func_t, 
		     m_free_func_t);

