/*
 * Copyright (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 *      ctmpl/list.c
 *      $Id: list.c,v 1.1.1.1 2004/01/03 15:41:41 hiddy Exp $
 *      general doubly linked list template
 *              written by Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 */

#include "ctmpl/memory.h"
#include "ctmpl/memory.c"

/* set iterator controller */
static CTMPL_INLINE void 
PKG_NAME_(list_set_ctl)(PKG_NAME_(list));
/*********************************************************/
	/***   External Interface ***/
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(list) 
PKG_NAME_(list_new)(void)
{
	return PKG_NAME_(list_new_by)(NULL, NULL);
}
CTMPL_STATIC CTMPL_INLINE PKG_NAME_(list) 
PKG_NAME_(list_new_by)(m_alloc_func_t af, 
		       m_free_func_t ff)
{

	PKG_NAME_(list) ret;

	/* default alloc/free function */
	if(!af)		af = PKG_NAME_(m_alloc);
	if(!ff)		ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "list", 
			  sizeof(struct PKG_NAME_(list)));
	ret->alloc = af;
	ret->free = ff;
	PKG_NAME_(list_set_ctl)(ret);
	return ret;
}
/*********************************************************/

#ifndef _CTMPL_LIST_C_
#define _CTMPL_LIST_C_
        /* these macro should be defined once */
#define l_headnode(x)        ((x)->head)
#define l_tailnode(x)        ((x)->tail)
#define l_nodecount(x)       (x)->count
#define l_isempty(x)         ((x)->count == 0)
#define ln_data(x)           ((x)->data)
#define ln_next(x)           ((x)->next)
#define ln_prev(x)           ((x)->prev)

#define CTMPL_LIST_FOREACH(L, N, V)     \
        for((N) = (L)->head; (N) ; (N) = (N)->next) \
                if(((V) = (N)->data) || 1)

#define LISTNODE_ALLOC(x) 	CTMPL_MALLOC(x->alloc, \
	PKG_NAME_STRING "listnode", sizeof(struct PKG_NAME_(listnode)))

#define LISTNODE_FREE(x, y)	CTMPL_FREE(x->free, \
					PKG_NAME_STRING "listnode", y)
#endif  /* _CTMPL_LIST_C_ */


/*********************************************************/
	/***   Internal Functions  ***/
static CTMPL_INLINE void 
PKG_NAME_(listnode_insert)(PKG_NAME_(list) l, 
			   PKG_NAME_(listnode) n,
			   PKG_NAME_(listdata_t) d)
{
	PKG_NAME_(listnode) newnode;
	ctmpl_assert(l != NULL);

	newnode = LISTNODE_ALLOC(l);
	ln_data(newnode) = d;
	
	/* if n == null, insert new data into head of list */
	if(!n){
		if(l_isempty(l)){
			l_headnode(l) = newnode;
			l_tailnode(l) = newnode;
		}
		else{
			l_headnode(l)->prev = newnode;
			ln_next(newnode) = l_headnode(l);
			l_headnode(l) = newnode;
		}
	}
	else { /* n != null */
		if(n == l_tailnode(l))
			l_tailnode(l) = newnode;
		else
			ln_next(n)->prev = newnode;	

		ln_prev(newnode) = n;
		ln_next(newnode) = ln_next(n);
		ln_next(n) = newnode;	
	}
	l_nodecount(l)++;
}

static CTMPL_INLINE PKG_NAME_(listdata_t) 
PKG_NAME_(listnode_remove)(PKG_NAME_(list) l, 
			   PKG_NAME_(listnode) n)
{
	PKG_NAME_(listdata_t) ret = ln_data(n);

	if(ln_next(n))
		ln_next(n)->prev = ln_prev(n);
	if(ln_prev(n))
		ln_prev(n)->next = ln_next(n);

	if(l_headnode(l) == n)
		l_headnode(l) = ln_next(n);

	if(l_tailnode(l) == n)
		l_tailnode(l) = ln_prev(n);
	
	l_nodecount(l)--;
	ctmpl_assert(l_nodecount(l) >= 0);

	LISTNODE_FREE(l, n);

	return ret;
}


/*********************************************************/
	/***   Aggregator Interfaces ***/

static CTMPL_INLINE void 
PKG_NAME_(list_delete_by)(PKG_NAME_(list) l, 
			  PKG_NAME_(listdata_access_func_t) f)
{
	aggregator_clear_by(l, f);
	CTMPL_FREE(l->free, PKG_NAME_STRING "list", l);
}

static CTMPL_INLINE PKG_NAME_(list) 
PKG_NAME_(list_dup_by)(PKG_NAME_(list) src, 
		       PKG_NAME_(listdata_duplicate_func_t) dup)
{
	PKG_NAME_(list) ret;
	register PKG_NAME_(listnode) ln;
	PKG_NAME_(listdata_t) val;

	ret = PKG_NAME_(list_new_by)(src->alloc, src->free);

	CTMPL_LIST_FOREACH(src, ln, val)
		iterator_append(ret, (dup? dup(val): val));
	return ret;
}

static CTMPL_INLINE void 
PKG_NAME_(list_clear_by)(PKG_NAME_(list) l, 
		  	 PKG_NAME_(listdata_access_func_t) f)
{
	PKG_NAME_(listnode) n, next;
	PKG_NAME_(listdata_t) val;
	
	ctmpl_assert(l);

	for(n = l_headnode(l); n ; n = next){
		next = ln_next(n);
		val = PKG_NAME_(listnode_remove)(l, n);
		/* if f is non-null, call delete function 
			for  data of each listnode  */
		if(f)	(*f)(val);	
	}
}
/*********************************************************/
	/***   Iterator Common Interfaces ***/
static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(list_count)(PKG_NAME_(list) l)
{
	return l_nodecount(l);
}

static void
PKG_NAME_(list_foreach)(PKG_NAME_(list) l, 
		        PKG_NAME_(listdata_foreach_func_t) func, 
			void *arg)
{
	register PKG_NAME_(listnode) ln;
	PKG_NAME_(listdata_t) val;
	if(func)
		CTMPL_LIST_FOREACH(l, ln, val)
			func(val, arg);
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_first)(PKG_NAME_(list) lp)
{
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = lp;
	iterator_item(ret) = l_headnode(lp);
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_last)(PKG_NAME_(list) lp)
{
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = lp;
	iterator_item(ret) = l_tailnode(lp);
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_at)(PKG_NAME_(list) lp, iterator_pos_t idx)
{
	int prev = 0;
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = lp;

	if(idx < 0){
		++prev;
		idx = -idx - 1;
	}

	if(idx >= l_nodecount(lp))
		iterator_item(ret) = NULL;
	else {
		PKG_NAME_(listnode) np = 
			(prev? l_tailnode(lp):l_headnode(lp));
		while(idx){
			np = (prev? ln_prev(np): ln_next(np)); 
			--idx;
		}
		iterator_item(ret) = np;
	}
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_next)(PKG_NAME_(list_iterator) itr)
{
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(!iterator_is_end(itr))
		iterator_item(ret) = ln_next(iterator_item(itr));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_prev)(PKG_NAME_(list_iterator) itr)
{
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(!iterator_is_end(itr))
		iterator_item(ret) = ln_prev(iterator_item(itr));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_erase_gonext)(PKG_NAME_(list_iterator) itr)
{
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	/* no item to erase */
	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = ln_next(iterator_item(itr));
	PKG_NAME_(listnode_remove)(iterator_aggregate(itr),
				   iterator_item(itr));
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_itr_erase_goprev)(PKG_NAME_(list_iterator) itr)
{
	PKG_NAME_(list_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	/* no item to erase */
	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = ln_prev(iterator_item(itr));
	PKG_NAME_(listnode_remove)(iterator_aggregate(itr),
				   iterator_item(itr));
	return ret;
}

static int 
PKG_NAME_(list_itr_is_equal)(PKG_NAME_(list_iterator) x,
                             PKG_NAME_(list_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static CTMPL_INLINE PKG_NAME_(listdata_t)
PKG_NAME_(list_itr_retrieve)(PKG_NAME_(list_iterator) itr)
{
	return ln_data(iterator_item(itr));
}

/*********************************************************/
   /***   Iterator Aggregator Specificic Interfaces ***/

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_find_between_by)(PKG_NAME_(list_iterator) beg,
				PKG_NAME_(list_iterator) end,
				PKG_NAME_(listdata_t) dat,
				PKG_NAME_(listdata_compare_func_t) cmp)
{
	PKG_NAME_(list_iterator) itr, ret;
	iterator_settype(ret, iterator_type(beg));
	iterator_subtype(ret) = iterator_subtype(beg);
	iterator_aggregate(ret) = iterator_aggregate(beg);
	iterator_item(ret) = NULL;
	
	itr = beg;

	while(!iterator_is_end(itr) &&
	      iterator_item(itr) != iterator_item(end)){
		PKG_NAME_(listdata_t) dat2 = iterator_data(itr);
		/* is same data? */
		if((cmp? !cmp(dat, dat2): dat == dat2))
			return itr;

		itr = iterator_next(itr);
	}
	/* not found */
	return ret;
}

static CTMPL_INLINE PKG_NAME_(list_iterator) 
PKG_NAME_(list_find)(PKG_NAME_(list) lp, 
		     PKG_NAME_(listdata_t) dat)
{
	return PKG_NAME_(list_find_between_by) (iterator_first(lp), 
					iterator_end(lp), dat, NULL);
}


static CTMPL_INLINE void 
PKG_NAME_(list_insert_after)(PKG_NAME_(list_iterator) itr, 
			     PKG_NAME_(listdata_t) v)
{
	PKG_NAME_(listnode_insert)(iterator_aggregate(itr), 
				   iterator_item(itr), v);
}

static CTMPL_INLINE void 
PKG_NAME_(list_insert_before)(PKG_NAME_(list_iterator) itr, 
			      PKG_NAME_(listdata_t) v)
{
	PKG_NAME_(listnode) n = iterator_item(itr);

	if(n) 	/* previous node */
		n = ln_prev(n);
	PKG_NAME_(listnode_insert)(iterator_aggregate(itr), n, v);
}

static CTMPL_INLINE void 
PKG_NAME_(list_swap)(PKG_NAME_(list_iterator) itr1, 
		     PKG_NAME_(list_iterator) itr2)
{
	PKG_NAME_(listdata_t) v;
	ctmpl_assert(!iterator_is_end(itr1) && !iterator_is_end(itr2));
	v = ln_data(iterator_item(itr1)); 
	ln_data(iterator_item(itr1)) = ln_data(iterator_item(itr2)) ;
	ln_data(iterator_item(itr2)) = v;
}

static CTMPL_INLINE PKG_NAME_(listdata_t) 
PKG_NAME_(list_assign)(PKG_NAME_(list_iterator) itr, 
		       PKG_NAME_(listdata_t) dat)
{
	ctmpl_assert(!iterator_is_end(itr));
	ln_data(iterator_item(itr)) = dat;
	return dat;
}

/*********************************************************/
	/* controller */
static PKG_NAME_(list_ctl) PKG_NAME_(list_ctl_sw) = 
{
	/* end iterator */
	{ -1, -1,  NULL, NULL },
	PKG_NAME_(list_delete_by),
	PKG_NAME_(list_clear_by),
	PKG_NAME_(list_dup_by),
	PKG_NAME_(list_foreach),
	PKG_NAME_(list_count),
	PKG_NAME_(list_itr_first),
	PKG_NAME_(list_itr_last),
	PKG_NAME_(list_itr_at),
	PKG_NAME_(list_itr_next),
	PKG_NAME_(list_itr_prev),
	PKG_NAME_(list_itr_erase_gonext),
	PKG_NAME_(list_itr_erase_goprev),
	PKG_NAME_(list_itr_is_equal),
	PKG_NAME_(list_itr_retrieve),
	PKG_NAME_(list_find),
	PKG_NAME_(list_find_between_by),
	PKG_NAME_(list_insert_before),
	PKG_NAME_(list_insert_after),
	PKG_NAME_(list_swap),
	PKG_NAME_(list_assign)
};


/* set iterator controller */
static CTMPL_INLINE void 
PKG_NAME_(list_set_ctl)(PKG_NAME_(list) lp)
{
	lp->ctl = &PKG_NAME_(list_ctl_sw);
}

