/*
 * Copyrigslm (C) 2001 and 2002 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>.
 * All rigslms reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyrigslm
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyrigslm
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "ctmpl/slist.c"

#ifndef _CTMPL_SLISTMAP_C_
#define _CTMPL_SLISTMAP_C_
#define slm_size(x)	((x)->)
#define slm_count(x)	((x)->count)
#define slm_isempty(x)	(slm_count(x) == 0)

#define slmn_key(t)	((t)->first)
#define slmn_value(t)	((t)->second)

#endif	/* _CTMPL_SLISTMAP_C_ */

static CTMPL_INLINE void PKG_NAME_(slm_set_ctl)(PKG_NAME_(slm));
static CTMPL_INLINE PKG_NAME_(slistnode) 
PKG_NAME_(slmslnode_get)(PKG_NAME_(slm) t, PKG_NAME_(slmkey_t) k);
static CTMPL_INLINE PKG_NAME_(slmnode) 
PKG_NAME_(slmnode_put)(PKG_NAME_(slm) t, PKG_NAME_(slmnode) n);


CTMPL_STATIC CTMPL_INLINE PKG_NAME_(slm) 
PKG_NAME_(slm_new)(PKG_NAME_(slm_cmpkey_func_t) ck)
{
	return PKG_NAME_(slm_new_by)(ck, NULL, NULL);
}

CTMPL_STATIC CTMPL_INLINE PKG_NAME_(slm) 
PKG_NAME_(slm_new_by)(PKG_NAME_(slm_cmpkey_func_t) cmpkey_func,
		      m_alloc_func_t af, 	
		      m_free_func_t ff)
{
	PKG_NAME_(slm) ret;
	ctmpl_assert(cmpkey_func != NULL);

        /* default alloc/free function */
	if(!af)		af = PKG_NAME_(m_alloc);
	if(!ff)		ff = PKG_NAME_(m_free);

	ret = CTMPL_MALLOC(af, PKG_NAME_STRING "slm", 
			sizeof(struct PKG_NAME_(slm)));
	ret->slist = PKG_NAME_(slist_new_by)
		((PKG_NAME_(listdata_compare_func_t))cmpkey_func, af, ff);
	ret->cmpkey = cmpkey_func;
	PKG_NAME_(slm_set_ctl)(ret);
	return ret;
}


static CTMPL_INLINE  struct PKG_NAME_(slm_iterator_data)
PKG_NAME_(slmnode_remove)(PKG_NAME_(slm) t, PKG_NAME_(slistnode) n)
{
	struct PKG_NAME_(slm_iterator_data) ret;

	PKG_NAME_(slmnode) slmn = (PKG_NAME_(slmnode))
		PKG_NAME_(listnode_remove)((PKG_NAME_(list))t->slist, n);

	ret.first = slmn_key(slmn);
	ret.second = slmn_value(slmn);

	CTMPL_FREE(((PKG_NAME_(list))t->slist)->free, 
			 PKG_NAME_STRING "slmnode", slmn);
	return ret;
}

static CTMPL_INLINE iterator_cnt_t 
PKG_NAME_(slm_count)(PKG_NAME_(slm) t)
{
	return sl_nodecount(t->slist);
}

static CTMPL_INLINE void
PKG_NAME_(slm_put)(PKG_NAME_(slm) t, 
		   PKG_NAME_(slmkey_t) k, 
		   PKG_NAME_(slmval_t) v)
{
	PKG_NAME_(slmnode) node;
	PKG_NAME_(slistnode) slnode;

	ctmpl_assert(t != NULL);
	
	/* check value whose key is same */
	slnode = PKG_NAME_(slmslnode_get)(t, k);

	/* value of same key already exists in table, swap */
	if(slnode){
#ifndef  PKG_USE_MULTI
		node = sln_data(slnode);
		slmn_key(node) = k;
		slmn_value(node) = v;
#else
		node = CTMPL_MALLOC(((PKG_NAME_(list))t->slist)->alloc, 
				    PKG_NAME_STRING "slmnode", 
				    sizeof(struct PKG_NAME_(slm_iterator_data)));
		slmn_key(node) = k;
		slmn_value(node) = v;
		PKG_NAME_(listnode_insert)((PKG_NAME_(list))(t->slist), 
					   sln_prev(slnode), node);
#endif /* PKG_USE_MULTI */
		return;
	}
	
	/* new node */
	node = CTMPL_MALLOC(((PKG_NAME_(list))t->slist)->alloc, 
			    PKG_NAME_STRING "slmnode", 
			    sizeof(struct PKG_NAME_(slm_iterator_data)));
	slmn_key(node) = k;
	slmn_value(node) = v;

	(void)PKG_NAME_(slmnode_put)(t, node); /* no node returns */
	return; 
}

static CTMPL_INLINE PKG_NAME_(slm_iterator)
PKG_NAME_(slm_get)(PKG_NAME_(slm) t, 
		   PKG_NAME_(slmkey_t) k)
{
	PKG_NAME_(slm_iterator) ret;
	iterator_set_forward(ret);
#ifndef  PKG_USE_MULTI
	iterator_subtype(ret) = 0;
#else 
	iterator_subtype(ret) = 1;
#endif /* PKG_USE_MULTI */
	iterator_aggregate(ret) = t;
	ctmpl_assert(t != NULL);

	iterator_item(ret) = PKG_NAME_(slmslnode_get)(t, k);
	return ret;
}

static CTMPL_INLINE void  
PKG_NAME_(slm_clear_by)(PKG_NAME_(slm) t,
		        PKG_NAME_(slmnode_access_func_t) df)
{
	PKG_NAME_(slist) sl;
	PKG_NAME_(listnode) ln, next; 
	struct PKG_NAME_(slm_iterator_data) idat;

	ctmpl_assert(t);

	sl = t->slist;
	for(ln = sl_headnode(sl); ln ; ln = next){
		next = ln->next;
		idat = PKG_NAME_(slmnode_remove)(t, ln);
		if(df)	(*df)(idat.first, idat.second);
	}
}

static CTMPL_INLINE void 
PKG_NAME_(slm_delete_by)(PKG_NAME_(slm) t, 
			 PKG_NAME_(slmnode_access_func_t) df)
{
	m_free_func_t free_func = ((PKG_NAME_(list))t->slist)->free;

	PKG_NAME_(slm_clear_by)(t, df);
	PKG_NAME_(slist_delete_by)(t->slist, NULL);
	CTMPL_FREE(free_func, PKG_NAME_STRING "slm", t);
}

static CTMPL_INLINE PKG_NAME_(slm) 
PKG_NAME_(slm_dup_by)(PKG_NAME_(slm) t, 
		      PKG_NAME_(slmnode_duplicate_func_t) f)
{
	PKG_NAME_(listnode) ln;
	PKG_NAME_(slmnode) slmn;
	PKG_NAME_(slm) ret;
	struct PKG_NAME_(slm_iterator_data) dup;

	ctmpl_assert(t != NULL);
	ret = PKG_NAME_(slm_new_by)(t->cmpkey, 
				    ((PKG_NAME_(list))t->slist)->alloc,
				    ((PKG_NAME_(list))t->slist)->free);

	CTMPL_SLIST_FOREACH(t->slist, ln, slmn){
		if(!f)
			PKG_NAME_(slm_put)(ret, slmn_key(slmn), slmn_value(slmn));	
		else {
			dup = f(slmn_key(slmn), slmn_value(slmn));
			PKG_NAME_(slm_put)(ret, dup.first, dup.second);	
		}
	}
	return ret;
}

static CTMPL_INLINE void  
PKG_NAME_(slm_foreach)(PKG_NAME_(slm) t,
		       PKG_NAME_(slmnode_foreach_func_t) func,
		       void *arg)
{
	PKG_NAME_(listnode) ln;
	PKG_NAME_(slmnode) slmn;
	ctmpl_assert(t != NULL);

	CTMPL_SLIST_FOREACH(t->slist, ln, slmn){
		if(func)
			func(slmn_key(slmn), slmn_value(slmn), arg);	
	}
}

static CTMPL_INLINE PKG_NAME_(slistnode) 
PKG_NAME_(slmslnode_get)(PKG_NAME_(slm) t, 
		      	 PKG_NAME_(slmkey_t) k)
{
	/* no check because of static function */
	PKG_NAME_(slist) sl = t->slist;
	PKG_NAME_(slistnode) n1;
	int cmp = 0;
	int pos, mid, left, rigslm, last;

	if(sl_isempty(sl))
		return NULL;

	left = 0;
	last = rigslm = sl_nodecount(sl) - 1;
	n1 = sl_headnode(sl);
	pos = 0;
	
	while(left < rigslm){
		mid = (left + rigslm) / 2;
		n1 = PKG_NAME_(slistnode_go)(n1, mid - pos);
		pos = mid;

		cmp = (*t->cmpkey)(slmn_key(sln_data(n1)), k);
		if(cmp < 0)
			left = mid + 1;
		else 
			rigslm = mid;
	}
	n1 = PKG_NAME_(slistnode_go)(n1, left - pos);
	cmp = (*t->cmpkey)(slmn_key(sln_data(n1)), k);
	if(!cmp)
		return n1;
	return NULL;
}

static CTMPL_INLINE PKG_NAME_(listdata_t) 
PKG_NAME_(slm_slistnode_add)(PKG_NAME_(slist) s, PKG_NAME_(listdata_t) v);

static CTMPL_INLINE PKG_NAME_(slmnode) 
PKG_NAME_(slmnode_put)(PKG_NAME_(slm) t, 
		       PKG_NAME_(slmnode) n)
{
	PKG_NAME_(slist) sl = t->slist;
	PKG_NAME_(slmnode) ret;	/* old node */

	ret = (PKG_NAME_(slmnode))PKG_NAME_(slm_slistnode_add)(sl, n);
	return ret;
}


static CTMPL_INLINE PKG_NAME_(listdata_t) 
PKG_NAME_(slm_slistnode_add)(PKG_NAME_(slist) s, 
			     PKG_NAME_(listdata_t) v)
{
	int cmp = 0;
	int pos, mid, left, rigslm, last;
	PKG_NAME_(list) lp;
	PKG_NAME_(listdata_t) ret = 0; 
	register PKG_NAME_(listnode) n1;
	PKG_NAME_(slm_cmpkey_func_t) compare = 
		(PKG_NAME_(slm_cmpkey_func_t))s->compare;
	PKG_NAME_(slmkey_t) key = slmn_key(v);

	ctmpl_assert(s != NULL);

	lp = (PKG_NAME_(list))s;

	if(sl_isempty(s) ||
	  (*compare)(key, slmn_key(sln_data(sl_headnode(s)))) < 0){
		iterator_prepend(lp, v);
		return NULL;
	}
	else if((*compare)(key, slmn_key(sln_data(sl_tailnode(s)))) > 0){
		iterator_append(lp, v);
		return NULL;
	}

	left = 0;
	last = rigslm = sl_nodecount(s) - 1;
	n1 = sl_headnode(s);
	pos = 0;
	
	while(left < rigslm){
		mid = (left + rigslm) / 2;
		n1 = PKG_NAME_(slistnode_go)(n1, mid - pos);
		pos = mid;

		cmp = (*compare)(slmn_key(sln_data(n1)), key);
		if(cmp < 0)
			left = mid + 1;
		else 
			rigslm = mid;
	}
	n1 = PKG_NAME_(slistnode_go)(n1, left - pos);
	cmp = (*compare)(slmn_key(sln_data(n1)), key);

	if(!cmp)/* same data entry */
		ctmpl_assert(!"must be checked before!");
	else {
		if(cmp > 0)
			n1 = sln_prev(n1);

		PKG_NAME_(listnode_insert)(lp, n1, v);
	}
	return ret;
}

static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_first)(PKG_NAME_(slm) slmp)
{
	PKG_NAME_(slm_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = slmp;
	iterator_item(ret) = sl_headnode(slmp->slist);
	return ret;
}


static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_last)(PKG_NAME_(slm) slmp)
{
	PKG_NAME_(slm_iterator) ret;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = slmp;
	iterator_item(ret) = sl_tailnode(slmp->slist);
	return ret;
}

static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_at)(PKG_NAME_(slm) slmp, iterator_pos_t idx) 
{
	PKG_NAME_(slm_iterator) ret;
	PKG_NAME_(slist_iterator) sitr;
	iterator_settype(ret, CTMPL_ITER_FORWARD);
	iterator_subtype(ret) = 0;
	iterator_aggregate(ret) = slmp;

	sitr = iterator_at(slmp->slist, idx);
	iterator_item(ret) = iterator_item(sitr);
	return ret;
}

static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_next)(PKG_NAME_(slm_iterator) itr)
{
	PKG_NAME_(slm) slmp = iterator_aggregate(itr);
	PKG_NAME_(slm_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = sln_next(iterator_item(itr)); 
	if(iterator_item(ret) && iterator_subtype(ret) &&
	   (*slmp->cmpkey)(slmn_key(sln_data(iterator_item(itr))), 
			   slmn_key(sln_data(iterator_item(ret)))) != 0)
		iterator_item(ret) = 0;
	return ret;
}

static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_prev)(PKG_NAME_(slm_iterator) itr)
{
	PKG_NAME_(slm) slmp = iterator_aggregate(itr);
	PKG_NAME_(slm_iterator) ret;
	iterator_settype(ret, iterator_type(itr));
	iterator_subtype(ret) = iterator_subtype(itr);
	iterator_aggregate(ret) = iterator_aggregate(itr);
	iterator_item(ret) = NULL;

	if(iterator_is_end(itr))
		return ret;

	iterator_item(ret) = sln_prev(iterator_item(itr)); 
	if(iterator_item(ret) && iterator_subtype(ret) &&
	   (*slmp->cmpkey)(slmn_key(sln_data(iterator_item(itr))), 
			   slmn_key(sln_data(iterator_item(ret)))) != 0)
		iterator_item(ret) = 0;
	return ret;
}

static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_erase_gonext)(PKG_NAME_(slm_iterator) itr)
{
	PKG_NAME_(slm_iterator) ret;

	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(slm_itr_next)(itr);
	PKG_NAME_(slmnode_remove)(iterator_aggregate(ret),
			  	  iterator_item(itr));
	return ret;
}

static PKG_NAME_(slm_iterator) 
PKG_NAME_(slm_itr_erase_goprev)(PKG_NAME_(slm_iterator) itr)
{
	PKG_NAME_(slm_iterator) ret;

	if(iterator_is_end(itr))
		return itr;

	ret = PKG_NAME_(slm_itr_prev)(itr);
	PKG_NAME_(slmnode_remove)(iterator_aggregate(ret),
			      	  iterator_item(itr));

	return ret;
}

static int 
PKG_NAME_(slm_itr_is_equal)(PKG_NAME_(slm_iterator) x,
                            PKG_NAME_(slm_iterator) y)
{ return CTMPL_ITR_IS_EQUAL(x, y); }


static PKG_NAME_(slm_iterator_data)
PKG_NAME_(slm_itr_retrieve)(PKG_NAME_(slm_iterator) itr)
{
	return (PKG_NAME_(slm_iterator_data))sln_data(iterator_item(itr));
}

static PKG_NAME_(slm_ctl) PKG_NAME_(slm_ctl_sw) = 
{
	{-1, -1, NULL, NULL},
	PKG_NAME_(slm_delete_by),
	PKG_NAME_(slm_clear_by),
	PKG_NAME_(slm_dup_by),
	PKG_NAME_(slm_foreach),
	PKG_NAME_(slm_count),
	
	PKG_NAME_(slm_itr_first),
	PKG_NAME_(slm_itr_last),
	PKG_NAME_(slm_itr_at),
	PKG_NAME_(slm_itr_next),
	PKG_NAME_(slm_itr_prev),
	PKG_NAME_(slm_itr_erase_gonext),
	PKG_NAME_(slm_itr_erase_goprev),
	PKG_NAME_(slm_itr_is_equal),
	PKG_NAME_(slm_itr_retrieve),
	PKG_NAME_(slm_get),
	PKG_NAME_(slm_put)
};

static CTMPL_INLINE void 
PKG_NAME_(slm_set_ctl)(PKG_NAME_(slm) slmp)
{
	slmp->ctl = &PKG_NAME_(slm_ctl_sw);
}

