#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "intmap.c"
#include "test_memory.h"
#define TBSIZE	11

int vopt = 0;
#define VPRINTF(x)	if(vopt)  printf x

unsigned int 	inthash(int i){ return i; }
int  		intcmp(const int a, const int b){ return a - b; }

void intprt(int key, int value){
	VPRINTF(("intprt(%d, %d)\n",  key, value));
}

void intprt2(int key, int value, void *arg){
	VPRINTF(("intprt2(%d, %d)\n",  key, value));
}

void printAll(intmap_iterator x){
	VPRINTF(("---- print all start(%s) ----\n",  
		(iterator_type(x) == CTMPL_ITER_FORWARD)?  
		"forward": "backward")); 
	while(!iterator_is_end(x)){
		VPRINTF(("(%d, %d) ,", iterator_data(x)->first,
				iterator_data(x)->second));
		x = iterator_next(x);
	}
	VPRINTF(("\n---- print all end(%s) ----\n",
		(iterator_type(x) == CTMPL_ITER_FORWARD)?  
		"forward": "backward")); 
};
	
void usage_exit(const char *name)
{
	fprintf(stderr, "%s [-dv] [-c count]\n", name);
	fprintf(stderr, "\t-c count:\tnumber of entrys(default: 1000).\n");
	fprintf(stderr, "\t-d:\t\tenable memory debug.\n");
	fprintf(stderr, "\t-v:\t\tprint debug message.\n");
	exit(1);
}

int main(int argc, char **argv)
{
	const char *progname = *argv;
	int i = 0, c, count = 1000;
	intmap aggr1, aggr2;
	intmap_iterator  itr;
	time_t t;
	int memdebug = 0;
	m_alloc_func_t	mal = NULL;
	m_free_func_t	mfr = NULL;

	while((c = getopt(argc, argv, "c:dv")) != EOF){
		switch(c){
		case 'c':
			count = atoi(optarg);
			if(count <= 0)
				usage_exit(progname);
			break;

		case 'd':
			++memdebug;
			break;
		case 'v':
			++vopt;
			break;
		default:
			usage_exit(progname);
		}
	}
	
	srand(time(&t));
	
	/* new aggregator */
	if(memdebug){
		mal = test_malloc;
		mfr = test_free;
	}
	aggr1 = intmap_new(TBSIZE, inthash, intcmp, mal, mfr);

	/* append data into aggregator*/
	VPRINTF(("put %d nodes into aggregator\n", count));
	while(1){
		int x = rand() % (count * 2);
		int y = rand() % (count * 2);
		iterator_insert_pair(aggr1,  x, y);
		VPRINTF(("insert pair: (%d, %d)\n", x, y));
		if(iterator_count(aggr1) == count)
			break;
	}

	/* access all list data */
	printAll(iterator_first(aggr1));

	/* access all list data in reverse order */
	itr = iterator_last(aggr1);
	iterator_set_backward(itr);
	printAll(itr);

	/* remove some list data */
	VPRINTF(("--  remove iterator at random\n"));
	for(i = -count/2; i < count/2 ; i++){
		/* delete */
		if((rand() % 3) == 0){
			itr = iterator_at(aggr1, i);
			if(iterator_is_end(itr)){
				if(i < 0) continue;
				else	  break;
			}
				
			VPRINTF(("erase node: (%d, %d)\n", 
				iterator_data(itr)->first,
				iterator_data(itr)->second));
			iterator_erase(itr);
		}
	}
	count = iterator_count(aggr1);
	
	/* print all nodes */
	printAll(iterator_first(aggr1));

	/* print all nodes in reverse order */
	itr = iterator_last(aggr1);
	iterator_set_backward(itr);
	printAll(itr);
	
	VPRINTF(("--  find all elements\n"));
	itr = iterator_first(aggr1);
	while(!iterator_is_end(itr)){
		intmap_iterator itr2 = iterator_find(aggr1, iterator_data(itr)->first);
		if(iterator_is_end(itr))
			printf("end iterator returns!!\n");
		else if(!iterator_is_equal(itr, itr2))
			printf("different iterator returns!!\n");
		itr = iterator_next(itr);
	}

	VPRINTF(("--  insert iterator at random\n"));
	for(i = 0; i < count/2 ; i++){
		if((rand() % 3) == 0){
			int key = rand() % (count * 2);
			int value = rand() % (count * 2);
			VPRINTF(("insert pair: (%d, %d)\n", key, value));
			iterator_insert_pair(aggr1, key, value);
		}
		itr = iterator_next(itr);
	}
	count = iterator_count(aggr1);

	/* print all nodes in forward/reverse order */
	printAll(iterator_first(aggr1));
	itr = iterator_last(aggr1);
	iterator_set_backward(itr);
	printAll(itr);

	VPRINTF(("--  test duplicate\n"));
	aggr2 = aggregator_dup(aggr1);
	/* clear & delete */
	VPRINTF(("--  erase all iterator of duplicate and delete \n"));
	itr = iterator_at(aggr2, 0);
	while(!iterator_is_end(itr))
		itr = iterator_erase(itr);
	assert(iterator_count(aggr2) == 0);
	aggregator_clear(aggr2);	/* remove all nodes */
	aggregator_delete(aggr2);	/* delete aggregator */

	/* delete list */
	VPRINTF(("--  clear/delete \n"));
	aggregator_clear_by(aggr1, intprt);
	assert(iterator_count(aggr1) == 0);
	aggregator_delete(aggr1);

	if(memdebug)
		memdump();
	return 0;
}
