#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#include <assert.h>
#include <time.h>
#include "test_memory.h"
#include "intseq.c"

int vopt = 0;

#define VPRINTF(x)	if(vopt)  printf x


int intcmp(const int a, const int b){ return a - b; }
void intprt(int a){ VPRINTF(("intprt(%d)\n", a)); }

void intprt2(int a, void *arg){ VPRINTF(("intprt(%d)\n", a)); }

void printAll(intseq_iterator itr)
{
	const char *str = (iterator_type(itr) == CTMPL_ITER_FORWARD? 
		"forward": "backward"); 
	VPRINTF(("---- print all start(%s) ----\n", str));
	while(!iterator_is_end(itr)){
		VPRINTF(("%d,", iterator_data(itr)));
		itr = iterator_next(itr);
	}
	VPRINTF(("\n---- print all end(%s) ----\n", str));
}
void usage_exit(const char *name)
{
	fprintf(stderr, "%s [-dv] [-c count]\n", name);
	exit(1);
}

int main(int argc, char **argv)
{
	const char *progname = *argv;
	int i = 0, c, count = 1000;
	intseq aggr1, aggr2;
	intseq_iterator  itr, itr2;
	int memdebug = 0;
	time_t t;
	m_alloc_func_t 	malc = NULL;
	m_free_func_t 	mfre = NULL;

	while((c = getopt(argc, argv, "c:dv")) != EOF){
		switch(c){
		case 'c':
			count = atoi(optarg);
			if(count <= 0)
				usage_exit(progname);
			break;

		case 'd':
			++memdebug;
			break;
		case 'v':
			++vopt;
			break;
		default:
			usage_exit(progname);
		}
	}
	
	srand(time(&t));
	
	if(memdebug){
		malc = test_malloc; 
		mfre = test_free;
	}
	
	/* new int list */
	VPRINTF(("create new sequence\n"));
	aggr1 = intseq_new(16, 16, malc, mfre);

	/* append data into list */
	VPRINTF(("put %d nodes into aggregator\n", count));
	for(i = 0; i < count; i++){
		int dat = rand();
		int prevcnt = iterator_count(aggr1);
		if(i % 2 == 0){
			VPRINTF(("iterator_append(%d)\n", dat));
			iterator_append(aggr1, dat);
		}
		else {
			VPRINTF(("iterator_prepend(%d)\n", dat));
			iterator_prepend(aggr1, dat);
		}
		assert(prevcnt + 1 == iterator_count(aggr1));
	}

	VPRINTF(("-- assign in order start\n"));
	for(i = 0; i < count; i++)
		iterator_assign_at(aggr1, i, i);
	VPRINTF(("-- assign in order end\n"));

	/* access all list data */
	printAll(iterator_first(aggr1));

	VPRINTF(("-- iterator_foreach start\n"));
	iterator_foreach(aggr1, intprt2, 0);
	VPRINTF(("-- iterator_foreach end\n"));

	/* remove some list data */
	VPRINTF(("delete some iterator at random\n"));
	itr = iterator_first(aggr1);
	for(i = 0; i < count/2 && !iterator_is_end(itr); i++){
		if((rand() % 3) == 0){
			VPRINTF(("erase data: %d\n",  
				 iterator_data(itr)));
			itr = iterator_erase(itr);
		}
		else 
			itr = iterator_next(itr);
	}

	
	/* print all nodes */
	printAll(iterator_first(aggr1));

	VPRINTF(("swap first and last\n"));
	iterator_swap(iterator_first(aggr1), iterator_last(aggr1));
	/* print all nodes in reverse order */
	itr = iterator_last(aggr1);
	iterator_set_backward(itr);
	printAll(itr);

	VPRINTF(("--  find all elements\n"));
	itr = iterator_first(aggr1);
	while(!iterator_is_end(itr)){
		itr2 = iterator_find_by(aggr1, iterator_data(itr), intcmp);
		if(iterator_is_end(itr2))
			printf("return end of iterator!\n");
		else if(!iterator_is_equal(itr, itr2))
			printf("some thing wrong!\n");
		itr = iterator_next(itr);
	}

	itr = iterator_at(aggr1, 0);
	VPRINTF(("--  insert iterator at random\n"));
	for(i = 0; i < count/2 && !iterator_is_end(itr) ; i++){
		if((rand() % 3) == 0){
			int r = rand();
			VPRINTF(("insert_after: %d\n",  r));
			iterator_insert_after(itr, r);
		}
		else if((rand() % 4) == 0){
			int r = rand();
			VPRINTF(("insert_after: %d\n",  r));
			iterator_insert_before(itr, r);
		}
		itr = iterator_next(itr);
	}
	printAll(iterator_first(aggr1));

	VPRINTF(("--  test duplicate\n"));
	aggr2 = aggregator_dup(aggr1);
	/* clear & delete */
	aggregator_clear(aggr2);	/* remove all nodes */
	aggregator_delete(aggr2);	/* delete aggregator */

	/* delete list */
	aggregator_delete_by(aggr1, intprt);

	if(memdebug)
		memdump();
	return 0;
}
