#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#include <assert.h>
#include <time.h>

#include "test_memory.h"
#include "intset.c"
#define HSTBLSIZE  13

int vopt = 0;
#define VPRINTF(x)	if(vopt)  printf x

void 
printAll(intset_iterator itr)
{
	const char *str = (iterator_type(itr) == CTMPL_ITER_FORWARD? 
		"forward": "backward"); 

	VPRINTF(("---- print all start(%s) ----\n", str));
	while(!iterator_is_end(itr)){
		VPRINTF(("%d,", iterator_data(itr)));
		itr = iterator_next(itr);
	}
	VPRINTF(("\n---- print all end(%s) ----\n", str));
}

unsigned int inthash(int i){ return (i < 0)? -i : i; }
int  intcmp(const int a, const int b){ return a - b; }
void intprt(int a){ VPRINTF(("intprt(%d)\n", a)); }
void intprt2(int a, void *arg){ VPRINTF(("intprt2(%d)\n", a)); }

void usage_exit(const char *name)
{
	fprintf(stderr, "%s [-dv] [-c count]\n", name);
	exit(1);
}

int main(int argc, char **argv)
{
	const char *progname = *argv;
	int i = 0, c, count = 1000;
	time_t t;
	int memdebug = 0;
	intset_iterator itr;
	intset  aggr, aggr2;
	m_alloc_func_t  malc = NULL;
	m_free_func_t   mfre = NULL;

	while((c = getopt(argc, argv, "c:dv")) != EOF){
		switch(c){
		case 'c':
			count = atoi(optarg);
			if(count <= 0)
				usage_exit(progname);
			break;

		case 'd':
			++memdebug;
			break;
		case 'v':
			++vopt;
			break;
		default:
			usage_exit(progname);
		}
	}
	
	srand(time(&t));
	
	/* new int list */
	if(memdebug){
		malc = test_malloc;
		mfre = test_free;
	}
	aggr = intset_new(13, inthash, intcmp, malc, mfre);

	/* append data into list */
	VPRINTF(("put %d nodes into aggregator\n", count));
	while(1){
		int x = rand() % (count * 2);
		iterator_insert(aggr, x);
		VPRINTF(("insert: %d\n", x));
		if(iterator_count(aggr) == count)
			break;
	}

	/* access all list data */
	printAll(iterator_first(aggr));

	VPRINTF(("--  foreach test start\n"));
	iterator_foreach(aggr, intprt2, NULL);
	VPRINTF(("--  foreach test end\n"));

	/* remove some list data */
	VPRINTF(("--  remove iterator at random\n"));
	itr = iterator_first(aggr);
	for(i = 0; i < count/2 && !iterator_is_end(itr); i++){
		if((rand() % 3) == 0){
			VPRINTF(("erase data: %d\n",  
				 iterator_data(itr)));
			itr = iterator_erase(itr);
		}
		else 
			itr = iterator_next(itr);
	}

	
	/* print all nodes */
	printAll(iterator_first(aggr));

	/* print all nodes in reverse order */
	itr = iterator_last(aggr);
	iterator_set_backward(itr);
	printAll(itr);

	VPRINTF(("--  find all elements\n"));
	itr = iterator_first(aggr);
	while(!iterator_is_end(itr)){
		intset_iterator itr2 = iterator_find(aggr, iterator_data(itr));
		if(iterator_is_end(itr2))
			printf("return end of iterator!\n");
		else if(!iterator_is_equal(itr, itr2))
			printf("some thing wrong!\n");
		itr = iterator_next(itr);
	}

	VPRINTF(("--  insert new data at random\n"));
	itr = iterator_at(aggr, 0);
	count = iterator_count(aggr);
	for(i = 0; i < count ; i++){
		if((rand() % 3) == 0){
			int r = rand() % (count * 2);
			VPRINTF(("insert: %d\n",  r));
			iterator_insert(aggr, r);
		}
		itr = iterator_next(itr);
	}

	printAll(iterator_first(aggr));

	VPRINTF(("--  test duplicate\n"));
	aggr2 = aggregator_dup(aggr);
	/* clear & delete */
	itr = iterator_at(aggr2, 0);
	while(!iterator_is_end(itr))
		itr = iterator_erase(itr);
	assert(iterator_count(aggr2) == 0);
	aggregator_clear(aggr2);	/* remove all nodes */
	aggregator_delete(aggr2);	/* delete aggregator */

	/* delete list */
	aggregator_delete_by(aggr, intprt);

	if(memdebug)
		memdump();
	return 0;
}
