/*
 *			dictbook.h
 * $Id: dictbook.h,v 1.6 2004/01/11 21:00:59 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _XP_BOOK_H_
#define _XP_BOOK_H_
#include "strlist.h"
#include "subr.h"

#define MAX_NFONTSIZE 		8

/* dictionary book structure */
struct dictbook {
	const char	*db_name;	/* official name */
	const char	*db_glid;	/* global identifier for this book */
	int		db_errno;	/* error number */
	char		db_flags;	/* flags */
	unsigned char	db_nfontsize;	/* number of font size */
	short		_db_pad;	/* padding */
	unsigned char db_fontsizelist[MAX_NFONTSIZE]; /* array of font size */

	/* open/close dictbook: success=1, fail=0 */
	int		(*db_open)(struct dictbook *);
	int		(*db_close)(struct dictbook *);

	/* free dictbook */
	void		(*db_free)(struct dictbook *);

	/* discard cache(ex. store cache into disk) */
	int		(*db_reclaim)(struct dictbook *);	

	/* lookup translation entries for the word
	 *    success: string-list,  no such entry: NULL(errno = 0),
	 *    error: NULL(errno != 0)
	 */
	strlist		(*db_lookup)(struct dictbook *, const char *word);
	strlist		(*db_getentry)(struct dictbook *, const char *word);

	/* get a bitmap entry indicated by the font id for the font size 
	 *    success: pointer to bitmap,  no such entry: NULL(errno = 0),
	 *    error: NULL(errno != 0)
	 */
	struct bitmap  *(*db_getbitmap)(struct dictbook *, const char *fontid, 
					int fontsizeidx);

	/* error string */
	const char 	*(*db_strerr)(struct dictbook *);
};

#define  DBFLAG_OPEN		0x1		/* book is open */
#define  DBFLAG_NOTAVAIL	0x2		/* book is not available */
#define  DBOOK_IS_OPEN(x)	((x)->db_flags & DBFLAG_OPEN)
#define  DBOOK_IS_AVAILABLE(x)	(((x)->db_flags & DBFLAG_NOTAVAIL) == 0)

#define  EDBNOTOPENED		0xabcdef	/* "Book is not opend." */
#define  EDBNOTAVAIL		0xabcdee	/* "Book is not available." */

/* external interfaces */
int 			dict_open(struct dictbook *dbp);
int 			dict_close(struct dictbook *dbp);
void 			dict_free(struct dictbook *dbp);
int			dict_reclaim(struct dictbook *dbp);
int			dict_error(struct dictbook *dbp);
const char *		dict_strerr(struct dictbook *dbp);

/* caller must free returned pointers */
strlist	dict_lookup(struct dictbook *dbp, const char *word);
strlist	dict_getentry(struct dictbook *dbp, const char *word);
/* caller must free returned bitmap */
struct bitmap 		*dict_getbitmap(struct dictbook *, const char *fondid, 
			    		unsigned int width,  unsigned int height);

#endif /* _XP_BOOK_H_ */

