/*
 *			dictlink.h
 * $Id: dictlink.h,v 1.7 2004/01/21 20:08:12 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _XP_DICLINK_H_
#define _XP_DICLINK_H_
struct whentry;

/* a list-node contains list of translation entries for the searched word */
struct dlistent {
	const char 	*dl_key;	/* translation entry id for dl_book */
	const char 	*dl_title;	/* title for translation entry */
	struct dictbook *dl_book;	/* dictionary book for this entry */
};


/* a node for each list/translation node */
struct dictnode {
	unsigned char		dn_type; /* DNTYPE_LIST, DNTYPE_TRANS */
	unsigned char		_dn_pad; /* padding */

	/* number of entris for list node or #th entry for each trans' node */
	unsigned short		dn_number; 

	const char 		*dn_key; /* searched word or entry id */

	struct dictnode 	*dn_parent; /* parent node */

	struct whentry 		*dn_whist;  /* lookup history */

	union {
		/* data for list-node */
		struct {
			/* list of results for the word */
			void			*_entlist;

			/* first entry of children */
			struct dictnode		*_first;
		} _lsdat;
		
		/* data for trans-node */
		struct {
			/* translation content */
			strlist  		_data; 

			/* a pointer to a book */
			struct dictbook		*_book;

			/* pointers to siblings */
			struct dictnode 	*_next, *_prev; 
		} _trdat;	/* translation data */
	} _dat;

	void			*dn_view;	/* manipulated by a user */
};

/* for a list node */
#define dn_entdata		_dat._lsdat._entlist
#define dn_entry		_dat._lsdat._first

/* for a translation node */
#define dn_content		_dat._trdat._data
#define dn_book			_dat._trdat._book
#define dn_next			_dat._trdat._next
#define dn_prev			_dat._trdat._prev

#define DNTYPE_LIST		0	/* a list of translation entiries */
#define DNTYPE_TRANS		1	/* translation entry */

/* init/finalize functions */
typedef void (*dictlink_error_handler_t)
	(const char *dictname, const char *errmsg, const char *cause);
						
void		dictlink_init(booklist, dictlink_error_handler_t); 
void		dictlink_test(booklist);
void		dictlink_fin(booklist);
void		dictlink_set_viewfree(void (*)(void *));

/* main controll interfaces */
/* movement interfaces: up/next/prev/quit */
struct dictnode *dl_up(booklist, struct dictnode *current);
struct dictnode *dl_next(booklist, struct dictnode *current);
struct dictnode *dl_prev(booklist, struct dictnode *current);
void		dl_quit(booklist, struct dictnode *current);
int		dl_has_up(struct dictnode *current);
int		dl_has_next(struct dictnode *current);
int		dl_has_prev(struct dictnode *current);

/* 
 * two search interfaces: "lookup" and "entry".
 *  "lookup" searches the "word" in the dictionaries indicated by
 * booklist. If one ore more entries are found, return dictnode.
 * Otherwise return NULL.
 *  "entry" searches a translation entry indicated by entry id "entry"
 * in a particular dictionary contained by current. If it is found,
 * return a corresponding node. Otherwise return NULL.
 *
 *  If property "ListView" is 0, dl_lookup() doesn't return 
 * any list-node but a first translation node of the list-node.
 */
struct dictnode *dl_lookup(booklist books, const char *word, 
			   struct dictnode *current);
struct dictnode *dl_entry(booklist books, const char *entry, 
			   struct dictnode *current);

/* interfaces for a list-node */
unsigned int 	dl_list_count(struct dictnode *current);
struct dlistent *dl_list_at(struct dictnode *current, int idx);

#endif /*  _XP_DICLINK_H_ */
