#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "bitmap.h"
#include "xwindow.h"
#include "memory.h"
#include "stk/stkbase.h"

/*
 *	stk string editor test.
 *
 *	top1 
 *	 |
 *    frame(200, 150)
 *	 |
 *    keyaction
 *	 |
 *     vlayout
 *	 |
 *	 +-- string
 *	 |      
 *	 |
 * 	 +-- editstr
 *
 */


void action_key(struct xactor *dst, xpoint dstpos, 
	    	xsignal sig, struct xsigarg *siginfo, void *actarg,
	    	struct xobject *top, xwindow *xwp)
{
	struct xs_keyinfo *kip = (struct xs_keyinfo *)siginfo;
	xobject  *estr_pos = (xobject *)actarg;
	xeditstr *estr = (xeditstr *)xpm_obj(estr_pos);

	if(xsignal_type(sig) != XSIG_KEYPRESS)
		return;

	if(strlen(kip->key) > 1){
		if(!strcasecmp(kip->key, "Return")){
			printf("\"%s\" selected!\n", estr->str);
			return;
		}
		else if(!strcasecmp(kip->key, "Backspace"))
			xeditstr_backspace(estr);
		else if(!strcasecmp(kip->key, "Delete"))
			xeditstr_delete(estr);
		else if(!strcasecmp(kip->key, "Left"))
			xeditstr_go_prev(estr);
		else if(!strcasecmp(kip->key, "Right"))
			xeditstr_go_next(estr);
		else if(!strcasecmp(kip->key, "Space"))
			xeditstr_putchar(estr, ' ');
		else if(!strcasecmp(kip->key, "period"))
			xeditstr_putchar(estr, '.');
	}
	else if(kip->modifier & ~(XKM_SHIFT | XKM_CAPS)){
		if(kip->modifier & ~XKM_CTRL)
			return;

		switch(*kip->key){
		case 'b':
			xeditstr_go_prev(estr);
			break;
		case 'f':
			xeditstr_go_next(estr);
			break;
		case 'd':
			xeditstr_delete(estr);
			break;
		case 'e':
			xeditstr_go_tail(estr);
			break;
		case 'a':
			xeditstr_go_top(estr);
			break;
		case 'k':
			xeditstr_kill_after(estr);
			break;
		case 'u':
			xeditstr_kill_before(estr);
			break;
		}
	}
	else 
		xeditstr_putchar(estr, *kip->key);

	xobj_redraw_object(top, estr_pos, xpm_x(estr_pos), xpm_y(estr_pos), xwp, 0);
}

int 
main(int argc, char **argv)
{
	xrootwindow *rwp = xrootwindow_open(NULL);
	xwindow *xwp = xwindow_create(rwp, 10, 10, 200, 150, 0, 0xffffff, 0, 0, 1);

	xproxy *proxy1;
	xobject *frame1;
	xobject *vlayout = xvlayout_new(200, 150, 0, XCRULE_CENTER);
	xobject *keyact;
	xeditstr *estr;
	xobject  *estr_pos;
	xattr *xatblack = xattr_alloc(rwp);
	xattr *xatwhite = xattr_alloc(rwp);
	xattr *xatblue = xattr_alloc(rwp);
	xattr *xatgreen = xattr_alloc(rwp);
	xattr *xatgray = xattr_alloc(rwp);


	stk_initialize(rwp, NULL, NULL);

	/* prepair colors */
	xattr_set_fgcolor(xatblack, 0x000000);
	xattr_set_bgcolor(xatblack, 0xffffff);
	xattr_set_font(xatblack, "fixed");
	xattr_set_fgcolor(xatwhite, 0xffffff);
	xattr_set_bgcolor(xatwhite, 0x000000);
	xattr_set_font(xatwhite, "fixed");
	xattr_set_fgcolor(xatblue, 0x0000ff);
	xattr_set_font(xatblue, "fixed");
	xattr_set_fgcolor(xatgreen, 0x00ff00);
	xattr_set_font(xatgreen, "fixed");
	xattr_set_fgcolor(xatgray, 0xd0d0d0);
	xattr_set_bgcolor(xatgray, 0xd0d0d0);
	xattr_set_font(xatgray, "fixed");

	/* edit string */
	estr = (xeditstr *)xeditstr_new(120, xatblue, xatwhite, XCRULE_CENTER);
	estr_pos = xposmark_new((xobject *)estr);

	xcont_add(vlayout, xexpander_new(), 0);
#define mkstr(c, a)     xstring_new(c, strlen(c), a, 0)
	xcont_add(vlayout, mkstr("String Editor Test", xatgreen), XCRULE_CENTER);
	xcont_add(vlayout, xexpander_new(), 0);
	xcont_add(vlayout, xcmpstpair_new(estr_pos, 
			xrect_new(xobj_width(estr) + 4, xobj_height(estr) + 4, 0, xatgray), 0),
			XCRULE_CENTER);
	xcont_add(vlayout, xexpander_new(), 0);

	/* vew1: frame1 */
	keyact = (xobject *)xkeyaction_new(vlayout);
	xact_register(keyact, XSIG_KEYPRESS, action_key, estr_pos, NULL);
	xact_register(keyact, XSIG_KEYRELEASE, action_key, estr_pos, NULL);

	frame1 = xframe_new(keyact, 200, 150);
	proxy1 = xproxy_new(xcmpcast(frame1));
	xproxy_accept(proxy1, XSIG_ANY);

	xeditstr_set(estr, "yogahe");

	stk_set(xwp, (xobject *)proxy1);
	xobj_dump(proxy1, stdout, 0);
	stk_map();
	stk_draw();

	stk_start_loop(NULL, NULL);

	stk_unmap();
	stk_finalize(rwp);

	xobj_free(proxy1);
	xattr_free(xatblack);
	xattr_free(xatblue);
	xattr_free(xatgray);
	xattr_free(xatgreen);

	xwindow_destroy(xwp);
	xrootwindow_close(rwp);

	stk_memdump();
	return 0;
}

