/*
 *			xascroll.c
 * $Id: xascroll.c,v 1.7 2004/01/23 02:45:27 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#define _XACTOR_INTERNAL

#include <stdio.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "xobject.h"
#include "xsignal.h"
#include "xactor.h"
#include "resource.h"

/*
 *    X Clickable Reactor Object 
 *	Treated signal: 
 *		- XSIG_ENTER
 *		- XSIG_LEAVE
 *		- XSIG_BTNPRESS
 *		- XSIG_BTNRELEASE
 *		- XSIG_GRABRELEASE
 */
struct xascroll {
	struct xactor 	xcl_base;
	struct xaction	*xcl_enter;
	struct xaction	*xcl_leave;
	struct xaction	*xcl_press;
	struct xaction	*xcl_release;
	struct xaction	*xcl_motion;
	struct xaction	*xcl_reset;
	struct xaction	*xcl_pageup;
	struct xaction	*xcl_pagedown;
	struct xaction	*xcl_pageleft;
	struct xaction	*xcl_pageright;
	xcursor		*xcl_cursor1;	/* save */
	xcursor		*xcl_cursor2;	/* save */
};


static void
xrfree(struct xobject *o)
{
	struct xascroll *c = (struct xascroll *)o;
	if(c->xcl_enter)
		xaction_delete(c->xcl_enter);
	if(c->xcl_leave)
		xaction_delete(c->xcl_leave);
	if(c->xcl_press)
		xaction_delete(c->xcl_press);
	if(c->xcl_release)
		xaction_delete(c->xcl_release);
	if(c->xcl_motion)
		xaction_delete(c->xcl_motion);
	if(c->xcl_reset)
		xaction_delete(c->xcl_reset);
	if(c->xcl_pageup)
		xaction_delete(c->xcl_pageup);
	if(c->xcl_pagedown)
		xaction_delete(c->xcl_pagedown);
	if(c->xcl_pageleft)
		xaction_delete(c->xcl_pageleft);
	if(c->xcl_pageright)
		xaction_delete(c->xcl_pageright);
	xobj_free(((struct xactor *)c)->xa_object);
	FREE("xascroll", c);
}

#define actdump(x, id, f, i, idt)  do {					\
	if(!x)								\
		fprintf(fp, "%s %s: no action registered.\n", idt, id); 	\
	else if(!(x)->xac_object)					\
		fprintf(fp, "%s %s: no object to be drawn.\n", idt, id); \
	else {								\
		fprintf(fp, "%s %s:\n", idt, id); 			\
		xobj_dump((x)->xac_object, fp, i + 1);			\
	}								\
} while(0)

static void
xrdump(struct xobject *o, FILE *fp, int off)
{
	struct xascroll *c = (struct xascroll *)o;
	struct xactor *r = (struct xactor *)o;
	char idt[1024], buf[1024], info[1024], *p = idt;

	xod_indent(idt, off);
	xobj_info(o, info, 1024);
	
	snprintf(buf, 1024, "%s%s<Scroller>: %s{\n", 
		 idt, xobj_typestr(o), info);
	fprintf(fp, "%s", buf);

	while(*p)
		*p++ = ' ';

	actdump(c->xcl_enter, "Enter", fp, off, idt); 
	actdump(c->xcl_leave, "Leave", fp, off, idt); 
	actdump(c->xcl_press, "BtnPrss", fp, off, idt); 
	actdump(c->xcl_release, "BtnReleaxe", fp, off, idt); 
	actdump(c->xcl_motion, "GrabMotion", fp, off, idt); 
	actdump(c->xcl_reset, "ScrolReset", fp, off, idt); 
	actdump(c->xcl_pageup, "PageUp", fp, off, idt); 
	actdump(c->xcl_pagedown, "PageDown", fp, off, idt); 
	actdump(c->xcl_pageleft, "PageLeft", fp, off, idt); 
	actdump(c->xcl_pageright, "PageRight", fp, off, idt); 

	fprintf(fp, "%s Main:\n", idt); 
	xobj_dump(r->xa_object, fp, off + 1);
	fprintf(fp, "%s}\n", idt);
}

static void
xrcompose(struct xcomposer *o)
{
	struct xactor *r = (struct xactor *)o;
	struct xascroll *c = (struct xascroll *)o;

	if(xobj_is_composer(r->xa_object))
		xcmp_compose(r->xa_object);
	if(xcmp_wvariable(o))
		xobj_width(o) = xobj_width(r->xa_object);
	if(xcmp_hvariable(o))
		xobj_height(o) = xobj_height(r->xa_object);

	xact_compose(c->xcl_enter); 
	xact_compose(c->xcl_leave); 
	xact_compose(c->xcl_press); 
	xact_compose(c->xcl_release);
	xact_compose(c->xcl_motion);
	xact_compose(c->xcl_reset);
	xact_compose(c->xcl_pageup);
	xact_compose(c->xcl_pagedown);
	xact_compose(c->xcl_pageleft);
	xact_compose(c->xcl_pageright);
}

static int
xracceptable(struct xactor *r, xsignal s)
{
	switch(xsignal_type(s)){
	case XSIG_ENTER:
	case XSIG_LEAVE:
	case XSIG_BTNPRESS:
	case XSIG_BTNRELEASE:
	case XSIG_GRABMOTION:
	case XSIG_SCRL_RESET:
	case XSIG_SCRL_PAGEUP:
	case XSIG_SCRL_PAGEDOWN:
	case XSIG_SCRL_PAGELEFT:
	case XSIG_SCRL_PAGERIGHT:
	case XSIG_SCRL_PAGESCRL:
		return 1;
	}
	return 0;

}

static struct xaction *
xrgetact(struct xactor *r, xsignal s)
{
	struct xascroll *c = (struct xascroll *)r;
	switch(xsignal_type(s)){
	case XSIG_ENTER:
		return c->xcl_enter;
	case XSIG_LEAVE:
		return c->xcl_leave;
	case XSIG_BTNPRESS:
		return c->xcl_press;
	case XSIG_BTNRELEASE:
		return c->xcl_release;
	case XSIG_GRABMOTION:
		return c->xcl_motion;
	case XSIG_SCRL_RESET:
		return c->xcl_reset;
	case XSIG_SCRL_PAGEUP:
		return c->xcl_pageup;
	case XSIG_SCRL_PAGEDOWN:
		return c->xcl_pagedown;
	case XSIG_SCRL_PAGELEFT:
		return c->xcl_pageleft;
	case XSIG_SCRL_PAGERIGHT:
		return c->xcl_pageright;
	}
	return NULL;
}

static int
xrregister(struct xactor *r, xsignal s, xaction_t a, 
	   void *p, struct xobject *o)
{
	struct xascroll *c = (struct xascroll *)r;
	struct xaction **dst = NULL;

	switch(xsignal_type(s)){
	case XSIG_ENTER:
		dst = &c->xcl_enter;
		break;
	case XSIG_LEAVE:
		dst = &c->xcl_leave;
		break;
	case XSIG_BTNPRESS:
		dst = &c->xcl_press;
		break;
	case XSIG_BTNRELEASE:
		dst = &c->xcl_release;
		break;
	case XSIG_GRABMOTION:
		dst = &c->xcl_motion;
		break;

	case XSIG_SCRL_RESET:
		dst = &c->xcl_reset;
		break;

	case XSIG_SCRL_PAGEUP:
		dst = &c->xcl_pageup;
		break;
	case XSIG_SCRL_PAGEDOWN:
		dst = &c->xcl_pagedown;
		break;
	case XSIG_SCRL_PAGELEFT:
		dst = &c->xcl_pageleft;
		break;
	case XSIG_SCRL_PAGERIGHT:
		dst = &c->xcl_pageright;
		break;
	}

	/* unacceptable signal */
	if(!dst)
		return 0;  /* fail */

	/* delete old action */
	if(*dst)
		xaction_delete(*dst);

	*dst = xaction_new(o, a, p);
	return 1;	/* success */
}

static void
xrreceive(struct xactor *act, xpoint pos, xsignal sig, struct xsigarg *sigarg, 
	  struct xobject *root, xwindow *w)
{
	struct xascroll *c = (struct xascroll *)act;
	struct xaction *a = NULL;
	struct stk_resource *rp = stk_get_resource();
	xcursor *cp;
	struct xs_btninfo *btp = (struct xs_btninfo *)sigarg;
	struct xs_bmotion *bmp = (struct xs_bmotion *)sigarg;

	switch(xsignal_type(sig)){
	case XSIG_ENTER:
		a = c->xcl_enter;
		if(xsignal_is_redirect(sig))
			return;
		/* swap x cursor */
		if(c->xcl_cursor1 == NULL){
			cp = xw_get_cursor(w);
			xw_set_cursor(w, rp->xc_handopen);
			c->xcl_cursor1 = cp;
		}
#ifdef STK_CURSOR_DEBUG		
		printf("s-enter-> 1:%x 2:%x\n", c->xcl_cursor1, c->xcl_cursor2);
#endif /* STK_CURSOR_DEBUG */
	
		break;

	case XSIG_LEAVE:
		a = c->xcl_leave;
		if(xsignal_is_redirect(sig))
			return;
		/* restore x cursor */
		if(c->xcl_cursor2)
			c->xcl_cursor2 = NULL;
		if(c->xcl_cursor1){
			xw_set_cursor(w, c->xcl_cursor1);
			c->xcl_cursor1 = NULL;
		}
#ifdef STK_CURSOR_DEBUG		
		printf("s-leave-> 1:%x 2:%x\n", c->xcl_cursor1, c->xcl_cursor2);
#endif /* STK_CURSOR_DEBUG */
		break;

	case XSIG_BTNPRESS:
		a = c->xcl_press;
		if(xsignal_is_redirect(sig))
			return;
		/* swap x cursor */
		if(btp->button == 0){
			cp = xw_get_cursor(w);
			xw_set_cursor(w, rp->xc_handgrab);

			if(c->xcl_cursor1)
				c->xcl_cursor2 = cp;
			/* button pressed without entering signal */
			else {
				c->xcl_cursor1 = cp;
				c->xcl_cursor2 = rp->xc_handopen;
			}
		}
#ifdef STK_CURSOR_DEBUG		
		printf("s-press-> 1:%x 2:%x\n", c->xcl_cursor1, c->xcl_cursor2);
#endif /* STK_CURSOR_DEBUG */
		break;

	case XSIG_BTNRELEASE:
		a = c->xcl_release;
		if(xsignal_is_redirect(sig))
			return;
		/* restore x cursor */
		if(c->xcl_cursor2){
			xw_set_cursor(w, c->xcl_cursor2);
			c->xcl_cursor2 = NULL;
		}
#ifdef STK_CURSOR_DEBUG		
		printf("s-release-> 1:%x 2:%x\n", c->xcl_cursor1, c->xcl_cursor2);
#endif /* STK_CURSOR_DEBUG */
		break;

	case XSIG_GRABMOTION:
		if(bmp->button != 0)
			return;

		/* enter with grabing button */
		if(!xsignal_is_redirect(sig) && c->xcl_cursor1 == NULL){
			cp = xw_get_cursor(w);
			xw_set_cursor(w, rp->xc_handgrab);
			c->xcl_cursor1 = cp;
			c->xcl_cursor2 = rp->xc_handopen;
#ifdef STK_CURSOR_DEBUG		
			printf("s-grab-> 1:%x 2:%x\n", c->xcl_cursor1, c->xcl_cursor2);
#endif /* STK_CURSOR_DEBUG */
		}

		xsc_scroll(act->xa_object, pos.x, pos.y, w, root, bmp->mx, bmp->my);
		break;

	case XSIG_SCRL_RESET:
		xsc_reset(act->xa_object, pos.x, pos.y, w, root);
		break;

	case XSIG_SCRL_PAGEUP:
		xsc_scroll(act->xa_object, pos.x, pos.y, w, root,
			   0, -xobj_height(act->xa_object));
		break;
	case XSIG_SCRL_PAGEDOWN:
		xsc_scroll(act->xa_object, pos.x, pos.y, w, root,
			   0, xobj_height(act->xa_object));
		break;
	case XSIG_SCRL_PAGELEFT:
		xsc_scroll(act->xa_object, pos.x, pos.y, w, root,
			   -xobj_width(act->xa_object), 0);
		break;
	case XSIG_SCRL_PAGERIGHT:
		xsc_scroll(act->xa_object, pos.x, pos.y, w, root,
			   xobj_width(act->xa_object), 0);
		break;
	case XSIG_SCRL_PAGESCRL:
		xsc_scroll(act->xa_object, pos.x, pos.y, w, root,
				bmp->mx, bmp->my);	
		break;
	}
}

struct xactor *
xascroll_new(struct xscroll *o)
{
	struct xascroll *c = MALLOC("xascroll", sizeof(struct xascroll));
	struct xactor *ret = (struct xactor *)c;

	xactor_init(ret, xobjcast(o), xactor_draw, xrfree, xrdump, 
		    xrcompose, xactor_find, xactor_relocate);

	ret->xa_acceptable = xracceptable;
	ret->xa_getaction = xrgetact;
	ret->xa_register = xrregister;
	ret->xa_receive = xrreceive;
	c->xcl_cursor1 = NULL;
	c->xcl_cursor2 = NULL;
	c->xcl_enter = NULL;
	c->xcl_leave = NULL;
	c->xcl_press = NULL;
	c->xcl_release = NULL;
	c->xcl_motion = NULL;
	c->xcl_reset = NULL;
	c->xcl_pageup = NULL;
	c->xcl_pagedown = NULL;
	c->xcl_pageleft = NULL;
	c->xcl_pageright = NULL;
	return ret;
}
