/*
 *			xobject.c
 * $Id: xobject.c,v 1.7 2004/01/20 02:55:33 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "xobject.h"
#include "xsignal.h"
#include "xactor.h"

#define xo_abs(x)	((x) < 0 ? -(x) : (x))
#define xo_max(x, y)	((x) < (y) ? (y) : (x))
#define xo_min(x, y)	((x) < (y) ? (x) : (y))

/*--------------------------------------------------------------*/
const char *
xod_typestr(int xp)
{
	switch(xp){
	/* primitive */
	case XOBJTYPE_EXPANDER:
		return "Expander";
	case XOBJTYPE_SPACE:
		return "Space";
	case XOBJTYPE_POINT:
		return "Point";
	case XOBJTYPE_LINE:
		return "Line";
	case XOBJTYPE_RECTANGLE:
		return "Rectangle";
	case XOBJTYPE_ARC:
		return "Arc";
	case XOBJTYPE_POLYGON:
		return "Polygon";
	case XOBJTYPE_STRING:
		return "String";
	case XOBJTYPE_WSTRING:
		return "WString";
	case XOBJTYPE_BITMAP:
		return "Bitmap";
	case XOBJTYPE_EDITSTR:
		return "String Editor";

	case XOBJTYPE_POSMARK:
		return "PositionMarker";
	case XOBJTYPE_ACTOR:
		return "Actor";
	case XOBJTYPE_VLAYOUT:
		return "VLayout";
	case XOBJTYPE_HLAYOUT:
		return "HLayout";
	case XOBJTYPE_COMPOSITE:
		return "Composite";
	case XOBJTYPE_CMPSTPAIR:
		return "Composite-pair";
	case XOBJTYPE_SELECTOR:
		return "Selector";
	case XOBJTYPE_SCROLL:
		return "Scroll";
	case XOBJTYPE_FRAME:
		return "Frame";
	}

	if(xp & XOBJTYPE_USER)
		return (xp & XOBJTYPE_COMPOSER)? 
				"User-defined Composer": "User-defined Object";
	return "Unknown Type";
}

int
xod_indent(char *bufp, int offset)
{
	int ret = 6;
	bufp[0] = '\0';
	strcpy(bufp, "(xobj)");
	
	while(offset > 0){
		if(offset == 1)
			strcat(bufp, "+-");
		else 
			strcat(bufp, "  ");
		--offset;
		ret += 2;
	}
	return ret;
}

int
xobj_info(struct xobject *xp, char *bufp, int size)
{
	if(xobj_is_actor(xp))
		return
		snprintf(bufp, size, "%s,%s(%c%dx%c%d),ref(%d)", 
			 xobj_is_enabled(xp)? "Enabled": "Disabled", 
			 xobj_is_active(xp)? "active": "inactive", 
			 xcmp_wvariable(xp)? 'v': 'f', xobj_width(xp), 
			 xcmp_hvariable(xp)? 'v': 'f', xobj_height(xp), 
			 xp->x_refcnt);

	return
	snprintf(bufp, size, "%s(%c%dx%c%d),ref(%d)", 
		 xobj_is_enabled(xp)? "Enabled": "Disabled", 
		 xcmp_wvariable(xp)? 'v': 'f', xobj_width(xp), 
		 xcmp_hvariable(xp)? 'v': 'f', xobj_height(xp), 
		 xp->x_refcnt);
}

/*--------------------------------------------------------------*/
