/*
 *			xprimitive.h
 * $Id: xprimitive.h,v 1.6 2004/01/20 02:55:34 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _XP_XPRIMITIVE_H_
#define _XP_XPRIMITIVE_H_

#define XPMI_INACTIVE_COLOR	0xd0d0d0

/* 
 *		xobject class hierarchy 
 *
 *	xobject*(*means not stub)
 * 	   +--- xprimitive*
 *	    	     +----- xexpander
 *	    	     +----- xspace
 *	    	     +----- xpoint
 *	    	     +----- xline
 *	    	     +----- xrectangle
 *	    	     +----- xarc
 *	    	     +----- xpolygon
 *	    	     +----- xstring
 *	    	     +----- xwstring
 *	    	     +----- xbitmap
 */

/* line object */
typedef struct xline {
	struct xprimitive _base;
	int 	x1, y1, x2, y2;
} xline;

/* rectangle object */
typedef struct xrect {
	struct xprimitive _base;
	int 	fill;		/* fill rectangle? */
} xrect;

/* arc */
typedef struct xarc {
	struct xprimitive _base;
	int 	fill;		/* fill rectangle? */
	int	angle1, angle2; /* angles */
} xarc;

/* bitmap */
typedef struct xbitmap {
	struct xprimitive _base;
	ximage 		*image;
	struct bitmap 	*bmp;
} xbitmap;

/* string */
typedef struct xstr {
	struct xprimitive _base;
	const char 	*str;
	short		nchars;
	char		bg;	/* draw background? */
	char		needfree;	/* need free? */
} xstr;

/* wide character string */
typedef struct xwstr {
	struct xprimitive _base;
	const char 	*str;
	short		nchars;
	char		bg;		/* draw background? */
	char		needfree;	/* need free? */
} xwstr;


/* string editing */
typedef struct xeditstr {
	struct xprimitive _base;
	xattr		  *xatrev;
	char 		  *str;
	short		  maxchar;
	short		  nchar;
	short		  pos;
	char		  layout;
	char		  _pad;
} xeditstr;

struct xobject *xeditstr_new(unsigned int width, xattr *at1, xattr *at2, 
			     unsigned char hlayout_rule);
void 		xeditstr_set(struct xeditstr *, const char *);
void 		xeditstr_reset(struct xeditstr *);
const char *	xeditstr_get(struct xeditstr *);
void 		xeditstr_putchar(struct xeditstr *, int c);
void 		xeditstr_backspace(struct xeditstr *);
void 		xeditstr_delete(struct xeditstr *);
void 		xeditstr_go_top(struct xeditstr *);
void 		xeditstr_go_next(struct xeditstr *);
void 		xeditstr_go_prev(struct xeditstr *);
void 		xeditstr_go_tail(struct xeditstr *);
void 		xeditstr_kill_before(struct xeditstr *);
void 		xeditstr_kill_after(struct xeditstr *);


#endif /* _XP_XPRIMITIVE_H_ */
