/*
 *                      xpaction.c
* $Id: xpaction.c,v 1.7 2004/01/25 02:39:36 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "dictbook.h"
#include "booklist.h"
#include "dictlink.h"
#include "strlist.h"
#include "strset.h"
#include "xobjcache.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "xpopdic_resource.h"
#include "xpopdic_component.h"
#include "xpopdic_control.h"
#include "xpstatus.h"
#include "xppage.h"
#include "xpaction.h"
#define mkstr(x,y)	xstring_na_new(x, strlen(x), y, 0)


static void 
xp_keyact_scroll(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  	 struct xsigarg *siginfo, void *actarg, 
	  	 struct xobject *top, xwindow *xwp, int move)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	struct xs_bmotion mov = {{ siginfo->x, siginfo->y}, 0, 0, move};
	int newsig, flags;
	int bodyx, bodyy;
	xpoint pos = {0, 0};
	xobject *xo;
	xactor *act;


	if(move == 0)
		newsig = XSIG_SCRL_RESET;	/* reset */
	else if(move == -1)
		newsig = XSIG_SCRL_PAGEUP; /* pageup */
	else if(move == 1)
		newsig = XSIG_SCRL_PAGEDOWN; /* pagedown */
	else 
		newsig = XSIG_SCRL_PAGESCRL;	/* scroll */

	/* must be a redirect signal to avoid cursor changing */
	xsignal_set_redirect(newsig);

	/* find scroll object in current cursor position. */
	flags = xcmpfind_build_flags(XCMPFIND_FIRSTMATCH, XOBJTYPE_ACTOR, 
				     xsignal_type(newsig));

	/* following position must be inside body */
	bodyx = (cp->xpr->width)/2;
	bodyy = (cp->xpr->height)/2;
	mov.pos.x= bodyx;
	mov.pos.y= bodyy;
	xo = xcmp_find(top, bodyx, bodyy, flags, &pos);

	if(!xo)	/* no scroll object here */	
		return;	

	act = (xactor *)xo;
	/* send signal to the object */
	xact_send_signal(act, pos, newsig, (struct xsigarg *)&mov, top, xwp);
}

static void 
xp_keyact_number(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  	 struct xsigarg *siginfo, void *actarg, 
	  	 struct xobject *top, xwindow *xwp, unsigned int number)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xobject		       *body = ((struct xppage *)cp->xo_cur)->xp_dep;
	int			newsig; 
	xobject		        *xo, *numx;
	struct xoposition 	*numpos;
	xactor			*act;


	if(!cp->dn_cur || cp->dn_cur->dn_type != DNTYPE_LIST)
		return;	/* this is not list node */

	newsig = (xsignal_type(sig) == XSIG_KEYPRESS)? 
				XSIG_BTNPRESS: XSIG_BTNRELEASE;

	/* set redirection flag */
	xsignal_set_redirect(newsig);

	/* to zero-origin */
	--number;

	if(number >= xcont_count(body))
		return;

	/* extracts hlayout out of body vlayout */
	numpos = xcont_at(body, number);
	xo = numpos->xp_object;

	/* 2nd object must be xposmark for clickable number */
	if(xcont_count(xo) < 2)
		return;
	numpos = xcont_at(xo, 1);	/* zero-origin */
	numx = numpos->xp_object;
	act = (xactor *)xpm_obj(numx);

	/* send signal! */
	xact_send_signal(act, xpm_pos(numx), newsig, siginfo, top, xwp);
}

/* 
 * 	keyboard actions 
 */
void 
xppage_keyaction(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  	 struct xsigarg *siginfo, void *actarg, 
	  	 struct xobject *top, xwindow *xwp)
{
	struct xs_keyinfo *kip = (struct xs_keyinfo *)siginfo;
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;

	if(strlen(kip->key) > 1){

		if(!strcasecmp(kip->key, "Return")){
			if(xsignal_type(sig) != XSIG_KEYRELEASE)
				return;
			xobj_deactivate(dst);
			xobj_enable(cp->xpc->xo_search);
			xobj_redraw_object(top, dst, dstpos.x, dstpos.y, xwp, 0);
		}
		if(!strcasecmp(kip->key, "Space")){
			if(xsignal_type(sig) == XSIG_KEYRELEASE)
				return;
			xp_keyact_scroll(dst, dstpos, sig, siginfo, actarg, 
	  	 			 top, xwp, 
					 (kip->modifier & XKM_SHIFT)? -1: 1);
		}
		if(!strcasecmp(kip->key, "Period")){
			if(xsignal_type(sig) == XSIG_KEYRELEASE)
				return;
			xp_keyact_scroll(dst, dstpos, sig, siginfo, actarg, 
	  	 			 top, xwp, 0);
		}
		return;
	}
	else if(kip->modifier & ~(XKM_SHIFT | XKM_CAPS)){
		if(kip->modifier & ~XKM_CTRL)
                        return;

		switch(*kip->key){
		case 'c':	/* ^C-c */
			/* quit */
			cp->do_quit = 1;
			if(xsignal_type(sig) != XSIG_KEYRELEASE)
				xpc_press_quit(cp);
			else
				xpc_release_quit(cp);
			break;
		case 'l':	/* ^C-l */
			/* redraw */
			xobj_redraw_object(top, dst, dstpos.x, dstpos.y, xwp, 0);
			break;

		case '0':	/* ^C-0 means 10 */
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			xp_keyact_number(dst, dstpos, sig, siginfo, actarg, 
	  	 			 top, xwp, *kip->key - '0' + 10);
			break;
		}
		return;
        }
        else {
		switch(*kip->key){
		case 'Q':
			cp->do_quit = 1;
		case 'q':
			if(xsignal_type(sig) != XSIG_KEYRELEASE)
				xpc_press_quit(cp);
			else
				xpc_release_quit(cp);
			break;
		case 'u':
			if(!cp->dn_cur || !dl_has_up(cp->dn_cur))
				return;
			if(xsignal_type(sig) != XSIG_KEYRELEASE)
				xpc_press_up(cp);
			else
				xpc_release_up(cp);
			break;
		case 'p':
			if(!cp->dn_cur || !dl_has_prev(cp->dn_cur))
				return;
			if(xsignal_type(sig) != XSIG_KEYRELEASE)
				xpc_press_prev(cp);
			else
				xpc_release_prev(cp);
			break;
		case 'n':
			if(!cp->dn_cur || !dl_has_next(cp->dn_cur))
				return;
			if(xsignal_type(sig) != XSIG_KEYRELEASE)
				xpc_press_next(cp);
			else
				xpc_release_next(cp);
			break;
		case 'j':
			if(xsignal_type(sig) == XSIG_KEYRELEASE)
				return;
			xp_keyact_scroll(dst, dstpos, sig, siginfo, actarg, 
	  	 			 top, xwp, 
					 xattr_get_font_height(cp->xpr->xatbody));
			break;
		case 'k':
			if(xsignal_type(sig) == XSIG_KEYRELEASE)
				return;
			xp_keyact_scroll(dst, dstpos, sig, siginfo, actarg, 
	  	 			 top, xwp, 
					 -xattr_get_font_height(cp->xpr->xatbody));
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			xp_keyact_number(dst, dstpos, sig, siginfo, actarg, 
	  	 			 top, xwp, *kip->key - '0');
			break;
		}
	}
}


/* 
 * 	icon button actions 
 */
static void 
quit_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_WARN, "Close..");
}
static void 
quit_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
quit_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_WARN, "Closing now");
	xpc_close(cp);
}
static void 
quit_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
}

static void 
next_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_LINK, "Goto the next entry");
}
static void 
next_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
next_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	/* do nothing */
}
static void 
next_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_UP, "Loading now...");
	xpc_next(cp);
}

static void 
prev_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_LINK, "Goto the previous entry");
}
static void 
prev_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
prev_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	/* do nothing */
}
static void 
prev_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_UP, "Loading now...");
	xpc_prev(cp);
}

static void 
up_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_UP, "Goto the up entry");
}
static void 
up_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
up_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	/* do nothing */
}
static void 
up_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	   struct xsigarg *siginfo, void *actarg, 
	   struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_UP, "Loading now...");
	xpc_up(cp);
}


/* actions for quit/next/prev/up buttons */
struct xp_caction iconacts[4] = {
	{ up_enter, up_leave, up_bpress, up_brelease }, 
	{ next_enter, next_leave, next_bpress, next_brelease }, 
	{ prev_enter, prev_leave, prev_bpress, prev_brelease }, 
	{ quit_enter, quit_leave, quit_bpress, quit_brelease }
};


/* clickable linked word */
static void 
wl_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xstr *xsp = (xstr *)dst->xa_object;
	snprintf(buf, 256, "Try to search \"%s\"...", xsp->str);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
}
static void 
wl_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
wl_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
}
static void 
wl_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xstr *xsp = (xstr *)dst->xa_object;
	snprintf(buf, 256, "Searching \"%s\" now...", xsp->str);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
	xpc_search(cp, xsp->str);
}
struct xp_caction wordact = { wl_enter, wl_leave, wl_bpress, wl_brelease};

/* clickable entry number on a top view */
static void 
en_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xstr *xsp = (xstr *)dst->xa_object;
	snprintf(buf, 256, "Goto entry #%s...", xsp->str);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
}
static void 
en_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
en_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
}
static void 
en_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xstr *xsp = (xstr *)dst->xa_object;
	snprintf(buf, 256, "Loading entry #%s now...", xsp->str);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
	xpc_go_at(cp, atoi(xsp->str) - 1);
}
struct xp_caction enumact = { en_enter, en_leave, en_bpress, en_brelease };

/* clickable entry title on a top view */
static void 
et_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xobject *xop = (xobject *)dst->xa_object;
	const char *entp = xpm_data(xop);
	snprintf(buf, 256, "Goto entry \"%s\"...", entp);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
}
static void 
et_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
et_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
}
static void 
et_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xobject *xop = (xobject *)dst->xa_object;
	const char *entp = xpm_data(xop);
	snprintf(buf, 256, "Loading entry \"%s\" now...", entp);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
	xpc_go_entry(cp, entp);
}
struct xp_caction etitleact = { et_enter, et_leave, et_bpress, et_brelease };
 

/* actions for an entry pointer */
static void 
ep_enter(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xobject *xop;
	xpoint pos;
	char buf[256];
	char entbuf[128], *p;

	/* 
	 *   entry pointer object is shared with all entry pointer. 
	 *  So, each entry id is stored in xposmark which is a parent 
	 *  object of the enter pointer object. 
	 *  Following code is to find the posmark object out.
	 */
	int flags = xcmpfind_build_flags(XCMPFIND_LASTMATCH, XOBJTYPE_POSMARK, 0);
	xop = xcmp_find(top, siginfo->x, siginfo->y, flags, &pos);

	if(xop == NULL || xobj_type(xop) != XOBJTYPE_POSMARK){
		fprintf(stderr, "entptr_find(): can't find out posmark for this pointer(%d, %d)\n", dstpos.x, dstpos.y);
		return;
	}

	strncpy(entbuf, (char *)xpm_data(xop), 128);
	if((p = index(entbuf, '>')) == NULL){
		fprintf(stderr, "entptr_find(): malformed entryid \"%s\".\n", entbuf);
		return;
	}
	*p = '\0';

	snprintf(buf, 256, "Goto entry \"%s\"...", entbuf);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
}

static void 
ep_leave(struct xactor *dst, xpoint dstpos, xsignal sig, 
	 struct xsigarg *siginfo, void *actarg, 
	 struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	xpc_send_status(cp, XSIG_STAT_NORMAL, NULL);
}
static void 
ep_bpress(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
}
static void 
ep_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	char buf[256];
	xobject *xop = (xobject *)dst->xa_object;
	xpoint pos;
	int flags = xcmpfind_build_flags(XCMPFIND_LASTMATCH, XOBJTYPE_POSMARK, 0);
	char entbuf[128], *p;

	xop = xcmp_find(top, siginfo->x, siginfo->y, flags, &pos);

	if(xop == NULL || xobj_type(xop) != XOBJTYPE_POSMARK){
		fprintf(stderr, "entptr_find(): can't find out posmark for this pointer(%d, %d)\n", dstpos.x, dstpos.y);
		return;
	}

	strncpy(entbuf, (char *)xpm_data(xop), 128);
	if((p = index(entbuf, '>')) == NULL){
		fprintf(stderr, "entptr_find(): malformed entryid \"%s\".\n", entbuf);
		return;
	}
	*p = '\0';

	snprintf(buf, 256, "Loading entry \"%s\" now...", entbuf);
	xpc_send_status(cp, XSIG_STAT_LINK, buf);
	xpc_go_entry(cp, entbuf);
}

struct xp_caction eptract = { ep_enter, ep_leave, ep_bpress, ep_brelease };

static void 
sc_brelease(struct xactor *dst, xpoint dstpos, xsignal sig, 
	  struct xsigarg *siginfo, void *actarg, 
	  struct xobject *top, xwindow *xwp)
{
	struct xpopdic_control *cp = (struct xpopdic_control *)actarg;
	struct xppage *curpage = (struct xppage *)cp->xo_cur;
	xobj_deactivate(curpage->xp_main);
	xobj_enable(cp->xpc->xo_search);
	xobj_redraw(top, 0, 0, xwp, 0);
}
xaction_t status_clicked = sc_brelease;
