/*
 *                      xppage.c
 * $Id: xppage.c,v 1.4 2004/01/22 12:49:28 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* 
 *	structure of xppage 
 *
 *   xppage
 *	|
 *   xcmpstpair
 *	|
 *	+---  xwp_search	<- default disabled but upper layar.
 *	|
 *	+---  xwp_main
 */

#include <stdio.h>
#include <string.h>
#include "memory.h"
#include "bitmap.h"
#include "xwindow.h"
#include "stk/stkbase.h"
#include "xppage.h"

#define xo_abs(x)	((x) < 0 ? -(x) : (x))
#define xo_max(x, y)	((x) < (y) ? (y) : (x))
#define xo_min(x, y)	((x) < (y) ? (x) : (y))

/*------------* Common Functions *-------------*/
static void 
xo_set(struct xobject *xp, int type, 
       unsigned int w, unsigned int h, 
       x_draw_t drawf, x_free_t freef, x_dump_t dumpf)
{
	xp->x_type = type;
	xp->x_flags = XOBJFLAG_ENABLE;
	xp->x_width = w;
	if(w == XC_VARIABLE)
		xp->x_flags |= XOBJFLAG_WVARIABLE;
		
	xp->x_height = h;
	if(h == XC_VARIABLE)
		xp->x_flags |= XOBJFLAG_HVARIABLE;

	xp->x_draw = drawf;
	xp->x_free = freef;
	xp->x_dump = dumpf;
	xp->x_refcnt = 0;
}

/*------------* X Position Marker *-------------*/
static void 
xppage_dump(struct xobject *xp, FILE *fp, int off)
{  
	int  n = 0;
	char buf[1024], info[1024];
	struct xppage *xcp = (struct xppage *)xp;

	n = xod_indent(buf, off);
	xobj_info(xp, info, 1024);
	snprintf(buf + n, 1024 - n, "XPopdic Page: %s", info);
	fprintf(fp, "%s\n", buf);

	xobj_dump(xcp->xp_obj, fp, off+1);
}


static void 
xppage_free(struct xobject *xp)
{ 
	struct xppage *xcp = (struct xppage *)xp;
	xobj_free(xcp->xp_obj);
	FREE("xppage", xp); 
}

static void
xppage_draw(struct xobject *xp, int x, int y, xwindow *xwp, 
	     int rx, int ry, unsigned int rw, unsigned int rh, int f)
{
	struct xppage *xcp = (struct xppage *)xp;
	xobj_draw_rect(xcp->xp_obj, x, y, xwp, rx, ry, rw, rh, f);
}

/* xppage dones't return itself if type is any */
static struct xobject *
xppage_find(struct xcomposer *xp, int x, int y, int f, xpoint *p)
{
	struct xppage *xcp = (struct xppage *)xp;
	if(xobj_is_composer(xcp->xp_obj))
		return xcmp_find(xcp->xp_obj, x, y, f, p);
	return NULL;
}

/* update position */
static void
xppage_relocate(struct xcomposer *xp, int x, int y)
{
	struct xppage *xcp = (struct xppage *)xp;
	if(xobj_is_composer(xcp->xp_obj))
		xcmp_relocate(xcp->xp_obj, x, y);
}

static void
xppage_compose(struct xcomposer *xp)
{
	struct xppage *xcp = (struct xppage *)xp;

	if(xobj_is_composer(xcp->xp_obj))
		xcmp_compose(xcp->xp_obj);

	/* if size variable, update size */
	if(xcmp_wvariable(xcp->xp_obj))
		xobj_width(xp) = xobj_width(xcp->xp_obj);
	if(xcmp_hvariable(xcp->xp_obj))
		xobj_height(xp) = xobj_height(xcp->xp_obj);
}

struct xobject *
xppage_new(xobject *main, xobject *search, xobject *dep)
{
	struct xppage *ret = MALLOC("xppage", sizeof(struct xppage));
	xobject       *xp;
	xproxy 	      *prp;
		
	xo_set(xobjcast(ret), XOBJTYPE_XPPAGE, 
	       xobj_width(main), xobj_height(main), 
	       xppage_draw, xppage_free, xppage_dump);

	/* set function pointers */
	xcmpcast(ret)->xc_compose = xppage_compose;
	xcmpcast(ret)->xc_find = xppage_find;
	xcmpcast(ret)->xc_relocate = xppage_relocate;

	xp = xframe_new(xcmpstpair_new(search, main, 0), 
				 xobj_width(main), xobj_height(main));
	prp = xproxy_new((xcomposer *)xp);
	xproxy_accept(prp, XSIG_ANY);
	ret->xp_obj = (xobject *)prp;
	xobj_incref(ret->xp_obj);

	/* data */
	ret->xp_main = main;
	ret->xp_search = search;
	ret->xp_dep = dep;
	return (struct xobject *)ret;
}

