/*
 *			xwindow.h
 * $Id: xwindow.h,v 1.6 2004/01/23 02:45:27 hiddy Exp $
 * Copyright (C) 2003 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 *               All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* 
 *		X Wrapper 
 */

#ifndef _XP_XWINDOW_H_
#define _XP_XWINDOW_H_
#include "bitmap.h"

/* 
 *  for somebody to manage all memory allocation.
 */
void xwrap_set_malloc(void *(*)(const char *, size_t, const char *, int),
		      void (*)(const char *, void *, const char *, int));

	/*  X Root Window structure */
typedef struct xrootwindow {
	int	xr_width; 	/* width of a screen */
	int	xr_height;	/* height of a screen */
} xrootwindow;

	/*  X Window structure */
typedef struct xwindow {
	int		    xw_type;  /* Window or Pixmap */
	struct xrootwindow *xw_root;
	int		    xw_width, xw_height;
	int		    xw_x, xw_y;
} xwindow;

#define XWTYPE_WINDOW		0
#define XWTYPE_PIXMAP		1

	/* X point for fill_polygon */
typedef struct xpoint {
	short x, y;
} xpoint;

	/* X Image wrapper */
typedef struct ximage { } ximage;

	/* X Cursor */
typedef struct xcursor {} xcursor;

	/*  X Window Attribute Structure */
typedef struct xattr { } xattr;
enum xattr_line_style { XatLineSolid, XatLineDoubleDash, XatLineOnOffDash };
enum xattr_cap_style { XatCapNotLast, XatCapButt, XatCapRound, XatCapProjecting };
enum xattr_join_style { XatJoinMiter, XatJoinRound, XatJoinBevel };
enum xattr_fill_style { XatFillSolid, XatFillTiled, 
			XatFillOpaqueStippled, XatFillStippled };
enum xattr_fill_rule { XatFillEvenOddRule, XatFillWindingRule };
enum xattr_arc_mode { XatArcPieSlice, XatArcChord };


/* function declarations for X root window */
xrootwindow *xrootwindow_open(const char *dispname);
void 	    xrootwindow_close(xrootwindow *);
void	    xroot_flush(xrootwindow *);
void	    xroot_getptr(xrootwindow *, int *x, int *y);
const char  *xroot_cutbuffer(xrootwindow *, int *nbytes);
void	    xroot_clear_cutbuffer(xrootwindow *);

/* control function declarations for X window */
xwindow *xwindow_create(xrootwindow *, int x, int y, unsigned int w, unsigned int h, 
			int bdwidth, long bgcolor, long bdcolor, 
			int shadow_enable, int winmgr_disable);
xwindow *xpixmap_create(xwindow *, long bgc, unsigned int w, unsigned int h);

void		xwindow_destroy(xwindow *);
void		xw_map(xwindow *);
void		xw_unmap(xwindow *);
void		xw_focus(xwindow *);
void		xw_focus_return(xwindow *);
void		xw_raise(xwindow *);
void		xw_clear(xwindow *);
void		xw_move(xwindow *, int x, int y);
void		xw_clear_region(xwindow *xp, int x, int y, 
				unsigned int w, unsigned int h);
void		xw_copy_region(xwindow *src, xwindow *dst, int srcx, int srcy,
			       unsigned int w, unsigned int h, int dstx, int dsty);
void		xw_set_cursor(xwindow *, xcursor *);
xcursor		*xw_get_cursor(xwindow *);
long		xw_get_bgcolor(xwindow *);
long		xw_get_bdcolor(xwindow *);

/* draw function declarations for X window */
void		xw_draw_point(xwindow *, struct xattr *, int x, int y);
void		xw_draw_line(xwindow *, struct xattr *, 
			     int x1, int y1, int x2, int y2);
void		xw_draw_rectangle(xwindow *, struct xattr *, int x, int y, 
				  unsigned int w, unsigned int h);
void		xw_draw_arc(xwindow *, struct xattr *, int x, int y, 
			    unsigned int w, unsigned int h, int angle1, int angle2);
void		xw_draw_image(xwindow *, struct xattr *, int x, int y,
			      struct ximage *, int w, int h);
void		xw_draw_string(xwindow *, struct xattr *, int x, int y,
			       const char *str, int size);
void		xw_draw_wstring(xwindow *, struct xattr *, int x, int y,
				const char *str, int size);
		/* these functions draws background indicated by xattr */
void		xw_draw_string_bg(xwindow *, struct xattr *, int x, int y,
				  const char *str, int size);
void		xw_draw_wstring_bg(xwindow *, struct xattr *, int x, int y,
				   const char *str, int size);
void		xw_fill_rectangle(xwindow *, struct xattr *, int x, int y, 
				  unsigned int w, unsigned int h);
void		xw_fill_arc(xwindow *, struct xattr *, int x, int y, 
			    unsigned int w, unsigned int h, int angle1, int angle2);
/*
void		xw_fill_polygon(xwindow *, struct xattr *, xpoint *, 
				int npoints, enum plyg_shape, enum plyg_mode);
*/


/* control function declarations for X window Attributes */
xattr		*xattr_alloc(xrootwindow *);
void		xattr_free(struct xattr *);

/* 
 * functions to set attributes into xattr structure 
 *  - return value: succeed = 1, fail = 0 
 */
int		xattr_set_font(xattr *, const char *);
int		xattr_get_font_width(xattr *);
int		xattr_get_str_width(xattr *, const char *, int size);
int		xattr_get_wstr_width(xattr *, const char *, int size);
int		xattr_get_font_height(xattr *);
int		xattr_set_fgcolor(xattr *, long color);
int		xattr_set_bgcolor(xattr *, long color);
int		xattr_get_fgcolor(xattr *, long *);
int		xattr_get_bgcolor(xattr *, long *);
int		xattr_set_line_width(xattr *, int);
int		xattr_set_line_style(xattr *, enum xattr_line_style);
int		xattr_set_cap_style(xattr *, enum xattr_cap_style);
int		xattr_set_join_style(xattr *, enum xattr_join_style);
int		xattr_set_fill_style(xattr *, enum xattr_fill_style);
int		xattr_set_fill_rule(xattr *, enum xattr_fill_rule);
int		xattr_set_arc_mode(xattr *, enum xattr_arc_mode);

/*
 * Declarations for funcion to manipulate x image 
 */
ximage		*ximage_from_bitmap(xrootwindow *, struct bitmap *);
void		ximage_free(ximage *);

/*
 * Declarations for funcion to manipulate x cursor
 */
xcursor		*xcursor_new(xrootwindow *xrw, struct bitmap *bmp,
			     struct bitmap *mask, int x_hot, int y_hot,
			     long fgcolor, long bgcolor);
void		xcursor_free(xrootwindow *xrw, xcursor *);

/* An event loop for X window */
struct xw_event_handler {
	/* 
	 * if each member function returns 0, xwindow_event_loop() stops 
	 * loop. 
	 */
	int 	(*xeh_button_pressed)(xwindow *, struct xw_event_handler *, 
				      int x, int y, int button);
	int 	(*xeh_button_released)(xwindow *, struct xw_event_handler *, 
				       int x, int y, int button);
	int 	(*xeh_button_motion)(xwindow *, struct xw_event_handler *, 
			      	     int x, int y, int button);
	int 	(*xeh_key_pressed)(xwindow *, struct xw_event_handler *, 
				   int x, int y, const char *p, int modifyer);
	int 	(*xeh_key_released)(xwindow *, struct xw_event_handler *, 
				    int x, int y, const char *p, int modifyer);
	int 	(*xeh_expose)(xwindow *, struct xw_event_handler *);
	int 	(*xeh_noexpose)(xwindow *, struct xw_event_handler *);
	int 	(*xeh_visibility_changed)(xwindow *, struct xw_event_handler *, 
					  int notify_type); /* to be impled */
	int 	(*xeh_mouse_in)(xwindow *, struct xw_event_handler *, int x, int y);
	int 	(*xeh_mouse_out)(xwindow *, struct xw_event_handler *, int x, int y);
	int 	(*xeh_focus_in)(xwindow *, struct xw_event_handler *);
	int 	(*xeh_focus_out)(xwindow *, struct xw_event_handler *);
};

/* key modifier */
#define XKM_SHIFT		0x1
#define XKM_CAPS		0x2
#define XKM_CTRL		0x4
#define XKM_META		0x8
#define XKM_BUTTON1		0x10
#define XKM_BUTTON2		0x20
#define XKM_BUTTON3		0x40

#if 0 /* doesn't work well in terms of running speed */
/* if one or more events are pending, return true */
int		xw_has_event(xwindow *, struct xw_event_handler *evp);

/* 
 *	xw_process_event(): processes pending events and return.
 *	 if there is no event, return 1. else if the event processr 
 *	returns 0, xw_process_event() return 0; 
 */
int		xw_process_event(xwindow *, struct xw_event_handler *);
#endif

/* 
 * event loop. if event handler returns 0, event_loop return. 
 *	- before/after are functions that are called before/after
 *	calling an event handler corresponding to a event.
 */
void		xw_event_loop(xwindow *, struct xw_event_handler *,
			      void (*before)(void), void (*after)(void));


/* Functions for X Selection */
typedef struct xselection {
	int		   xsl_sel;	/* 0: PRIMARY, 1: SECONDARY */
} xselection;
#define  XSEL_PRIMARY		0
#define  XSEL_SECONDARY		1

struct xselection *xselection_new(struct xrootwindow *xsl_root, 
				  int sel);
void 		   xselection_free(struct xselection *);
int 		   xselection_get(struct xselection *, char *buf, int len);
/* set function is not implemented */

#endif /* _XP_XWINDOW_H_ */
